----------------------------------------------------------------------------------


----------------------------------------------------------------------------------




-- Filename     Arbiter.vhd




-- Title        Arbiter for PtrnGen and DataCmpr




--




-- Company      Design Gateway Co., Ltd.




-- Project      JAMPII




-- PJ No.       RD0505




-- Syntax       VHDL




-- Note         




--




--




-- Version      1.00




-- Author       P.Kosol




-- Date         2008/09/11




-- Remark       New Creation




----------------------------------------------------------------------------------

----------------------------------------------------------------------------------
Library	IEEE;
Use		IEEE.STD_LOGIC_1164.all;
Use		IEEE.STD_LOGIC_UNSIGNED.all;

Entity Arbiter is
Port(
		Clk			: in	std_logic;
		RstB		: in	std_logic;
		
		CmdReq		: out	std_logic;
		Cmd			: out	std_logic;
		Busy		: in	std_logic;
		Addr		: out	std_logic_vector(14 downto 0);
		
		CmdReq1		: in	std_logic;
		Cmd1		: in	std_logic;
		Busy1		: out	std_logic;
		Addr1		: in	std_logic_vector(14 downto 0);
		
		CmdReq2		: in	std_logic;
		Cmd2		: in	std_logic;
		Busy2		: out	std_logic;
		Addr2		: in	std_logic_vector(14 downto 0)
	);
End Entity Arbiter;

Architecture rtl of Arbiter is

------------------------------------------------------------------------------
-- Constant Declaration
------------------------------------------------------------------------------
	
	type state is(
					stIdle,
					stWtBusy,
					stWtIdle
				);
				
	signal	rState		: state;
	
------------------------------------------------------------------------------
-- Signal Declaration
------------------------------------------------------------------------------
	
	signal	rCmdReq		: std_logic;
	signal	rCmd		: std_logic;
	signal	rBusy1		: std_logic;
	signal	rBusy2		: std_logic;
	signal	rAddr		: std_logic_vector(14 downto 0);
	signal	rChSel		: std_logic;
	
Begin

------------------------------------------------------------------------------
-- Output Assignment
------------------------------------------------------------------------------
	
	CmdReq	<= rCmdReq;
	Cmd		<= rCmd;
	Busy1	<= rBusy1;
	Busy2	<= rBusy2;
	Addr	<= rAddr;
	
------------------------------------------------------------------------------
-- DFF
------------------------------------------------------------------------------

	u_rState : Process (RstB, Clk) Is
	Begin
		if ( RstB='0' ) then
			rState	<= stIdle;
		elsif ( rising_edge(Clk) ) then
			case (rState) is			
				
				when stIdle		=>
					if ( Busy='0' ) then
						if ( CmdReq2='1' or CmdReq1='1' ) then
							rState	<= stWtBusy;
						else
							rState	<= stIdle;
						end if;
					else
						rState	<= stIdle;
					end if;
				
				when stWtBusy	=>
					if ( Busy='1' ) then
						rState	<= stWtIdle;
					else
						rState	<= stWtBusy;
					end if;
					
				when stWtIdle		=>
					if ( Busy='0' ) then
						rState	<= stIdle;
					else
						rState	<= stWtIdle;
					end if;
					
				when others		=>
					rState		<= stIdle;
									
			end case;				
		end if;
	End Process u_rState;

	u_rChSel : Process (RstB, Clk) Is
	Begin
		if ( RstB='0' ) then
			rChSel		<= '1';
		elsif ( rising_edge(Clk) ) then
			if ( rState=stIdle and Busy='0' ) then
				if ( CmdReq2='1' ) then
					rChSel	<= '1';
				elsif ( CmdReq1='1' ) then
					rChSel	<= '0';
				else
					rChSel	<= rChSel;
				end if;
			else
				rChSel	<= rChSel;
			end if;
		end if;
	End Process u_rChSel;

	u_rCmdReq : Process (RstB, Clk) Is
	Begin
		if ( RstB='0' ) then
			rCmdReq		<= '0';
		elsif ( rising_edge(Clk) ) then
			if ( Busy='1' ) then
				rCmdReq		<= '0';
			elsif ( rState=stWtBusy ) then
				rCmdReq		<= '1';
			else
				rCmdReq		<= rCmdReq;
			end if;
		end if;
	End Process u_rCmdReq;
	
	u_rBusy1 : Process (RstB, Clk) Is
	Begin
		if ( RstB='0' ) then
			rBusy1		<= '1';
		elsif ( rising_edge(Clk) ) then
			if ( (rState=stWtBusy or rState=stWtIdle) and rChSel='0' ) then
				rBusy1	<= Busy;
			else
				rBusy1	<= '0';
			end if;
		end if;
	End Process u_rBusy1;

	u_rBusy2 : Process (RstB, Clk) Is
	Begin
		if ( RstB='0' ) then
			rBusy2		<= '1';
		elsif ( rising_edge(Clk) ) then
			if ( (rState=stWtBusy or rState=stWtIdle) and rChSel='1' ) then
				rBusy2	<= Busy;
			else
				rBusy2	<= '0';
			end if;			
		end if;
	End Process u_rBusy2;

	rCmd	<= rChSel;	
	
	u_rAddr : Process (RstB, Clk) Is
	Begin
		if ( RstB='0' ) then
			rAddr	<= (others => '0');
		elsif ( rising_edge(Clk) ) then
			if ( rChSel='0' ) then
				rAddr	<= Addr1;
			else
				rAddr	<= Addr2;
			end if;				
		end if;
	End Process u_rAddr;
	
End Architecture rtl;		
	