----------------------------------------------------------------------------------

----------------------------------------------------------------------------------


-- Filename     ConfigStorage.vhd


-- Title        ConfigStorage


--


-- Company      Design Gateway Co., Ltd.


-- Project      JAMPII


-- PJ No.       RD0505


-- Syntax       VHDL


-- Note         


--


--


-- Version      1.00


-- Author       P.Kosol


-- Date         2008/10/06


-- Remark       New Creation


----------------------------------------------------------------------------------

----------------------------------------------------------------------------------
Library	IEEE;
Use		IEEE.STD_LOGIC_1164.all;
Use		IEEE.STD_LOGIC_ARITH.all;

Entity ConfigStorage is
	Port (
		Clk    		: in 	std_logic;
		RstB 		: in 	std_logic;
				
		UserBusy	: in	std_logic;	
		UserStart	: out	std_logic;
		UserDevAddr	: out	std_logic_vector( 6 downto 0 );
		UserRegAddr	: out	std_logic_vector( 7 downto 0 );
		UserData	: out	std_logic_vector(15 downto 0 );
		
		ConfigRdy	: out	std_logic	
	);
End Entity ConfigStorage;

Architecture rtl of ConfigStorage is
----------------------------------------------------------------------------------
-- Constant Declaration
----------------------------------------------------------------------------------
	constant	CMD			: std_logic_vector(23 downto 0 ) := x"0C2000";-- Command Register		
--	constant	QPR			: std_logic_vector(23 downto 0 ) := x"C49E30";--***
	constant	QPR			: std_logic_vector(23 downto 0 ) := x"C49030";-- VGA Pass Through Mode
	
	constant	BYPASS		: std_logic_vector(23 downto 0 ) := x"0A0000";--***
	constant	CMD_SRST	: std_logic_vector(23 downto 0 ) := x"0C2001";-- Soft Reset
	--constant	CMD			: std_logic_vector(23 downto 0 ) := x"0C2000";--
	
	constant	NCON_L		: std_logic_vector(23 downto 0 ) := x"100000";-- Set NCON_L
	constant	NCON_H		: std_logic_vector(23 downto 0 ) := x"120000";-- Set NCON_H
	constant	NCOD_L		: std_logic_vector(23 downto 0 ) := x"140000";-- Set NCOD_L
	constant	NCOD_H		: std_logic_vector(23 downto 0 ) := x"160000";-- Set NCOD_H
			
--	constant	PLL_M		: std_logic_vector(23 downto 0 ) := x"1831E3";--***
--	constant	PLL_N		: std_logic_vector(23 downto 0 ) := x"1A0086";--***
--	constant	PLL_P		: std_logic_vector(23 downto 0 ) := x"1C0101";--***
	constant	PLL_M		: std_logic_vector(23 downto 0 ) := x"1813DE";-- Set PLL_M (M=1007)
	constant	PLL_N		: std_logic_vector(23 downto 0 ) := x"1A010D";-- Set PLL_N (N=270)
	constant	PLL_P		: std_logic_vector(23 downto 0 ) := x"1C0303";-- Set PLL_P (P=4)

	constant	CMD_LPLL	: std_logic_vector(23 downto 0 ) := x"0C2002";-- Load PLL values
	--constant	CMD			: std_logic_vector(23 downto 0 ) := x"0C2000";-- 
	
--	constant	MISC		: std_logic_vector(23 downto 0 ) := x"0E0113";--***
	constant	MISC		: std_logic_vector(23 downto 0 ) := x"0E0003";-- N888 RGB Port Mapping Mode
	constant	VID_CTRL	: std_logic_vector(23 downto 0 ) := x"920006";-- VGA-RGB Video O/P Mode					 
	
	constant	MISC47		: std_logic_vector(23 downto 0 ) := x"470000";
	
	type State is (
					stIdle,
					stCmd1,
					stQpr,
					
					stBypass,--***
					
					stCmdSrst,
					stCmd2,
					
					stNconL,--***
					stNconH,--***
					stNcodL,--***
					stNcodH,--***					
					
					stPllM,
					stPllN,
					stPllP,
					stCmdLpll,
					stCmd3,
					stMisc,
					stVidCtrl,
					
					stMisc47,
					
					stEnd
				);
	signal	rState			: State;
							
----------------------------------------------------------------------------------
-- Signal Declaration
----------------------------------------------------------------------------------
	signal	rUserBusy		: std_logic;
	signal	rUserStart		: std_logic;
	signal	rUserRegAddr	: std_logic_vector( 7 downto 0 );
	signal	rUserData		: std_logic_vector(15 downto 0 );
	signal	rConfigRdy		: std_logic;
	
Begin
----------------------------------------------------------------------------------
-- Output Assignment
----------------------------------------------------------------------------------	
	UserStart	<=	rUserStart;	
	UserDevAddr	<=	"1101010";
	UserRegAddr	<=	rUserRegAddr;
	UserData	<=	rUserData;
	ConfigRdy	<=	rConfigRdy;	

----------------------------------------------------------------------------------
-- DFF
----------------------------------------------------------------------------------
	u_rState : Process (RstB, Clk) Is
	Begin
		if ( RstB='0' ) then
			rState	<= stIdle;
		elsif ( rising_edge(Clk) ) then
			case rState is
			
				when stIdle			=>
					if ( UserBusy='0' and rUserBusy='1' ) then				  
						rState	<= stCmd1;								  
					else													  
						rState	<= stIdle;									  
					end if;													  
																			  
				when stCmd1			=>										  
					if ( UserBusy='0' and rUserBusy='1' ) then				  
						rState	<= stQpr;								  
					else													  
						rState	<= stCmd1;								  
					end if;													  
																			  
				when stQpr			=>										  
					if ( UserBusy='0' and rUserBusy='1' ) then				  
						--rState	<= stCmdSrst;								  
						rState	<= stByPass;	
					else													  
						rState	<= stQpr;
					end if;
										

				-----------------------------------------------
				when stBypass			=>										  
					if ( UserBusy='0' and rUserBusy='1' ) then				  
						rState	<= stCmdSrst;								  
					else													  
						rState	<= stBypass;
					end if;
				-----------------------------------------------


				when stCmdSrst		=>
					if ( UserBusy='0' and rUserBusy='1' ) then
						rState	<= stCmd2;
					else
						rState	<= stCmdSrst;
					end if;
					
				when stCmd2			=>
					if ( UserBusy='0' and rUserBusy='1' ) then
						rState	<= stPllM;
						--rState	<= stNconL;
					else
						rState	<= stCmd2;
					end if;


--				-----------------------------------------------
--				when stNconL		=>
--					if ( UserBusy='0' and rUserBusy='1' ) then
--						rState	<= stNconH;
--					else
--						rState	<= stNconL;
--					end if;
--				when stNconH		=>
--					if ( UserBusy='0' and rUserBusy='1' ) then
--						rState	<= stNcodL;
--					else
--						rState	<= stNconH;
--					end if;
--				when stNcodL		=>
--					if ( UserBusy='0' and rUserBusy='1' ) then
--						rState	<= stNcodH;
--					else
--						rState	<= stNcodL;
--					end if;
--				when stNcodH		=>
--					if ( UserBusy='0' and rUserBusy='1' ) then
--						rState	<= stPllM;
--					else
--						rState	<= stNcodH;
--					end if;
--				-----------------------------------------------	
					
				
				when stPllM			=>
					if ( UserBusy='0' and rUserBusy='1' ) then
						rState	<= stPllN;
					else
						rState	<= stPllM;
					end if;

				when stPllN			=>
					if ( UserBusy='0' and rUserBusy='1' ) then
						rState	<= stPllP;
					else
						rState	<= stPllN;
					end if;

				when stPllP			=>
					if ( UserBusy='0' and rUserBusy='1' ) then
						rState	<= stCmdLpll;
					else
						rState	<= stPllP;
					end if;

				when stCmdLpll		=>
					if ( UserBusy='0' and rUserBusy='1' ) then
						rState	<= stCmd3;
					else
						rState	<= stCmdLpll;
					end if;

				when stCmd3			=>
					if ( UserBusy='0' and rUserBusy='1' ) then
						rState	<= stMisc;
					else
						rState	<= stCmd3;
					end if;

				when stMisc			=>
					if ( UserBusy='0' and rUserBusy='1' ) then
						rState	<= stVidCtrl;
					else
						rState	<= stMisc;
					end if;
				
				when stVidCtrl		=>
					if ( UserBusy='0' and rUserBusy='1' ) then
						rState	<= stEnd;
						--rState	<= stMisc47;
					else
						rState	<= stVidCtrl;
					end if;
				


--				when stMisc47		=>
--					if ( UserBusy='0' and rUserBusy='1' ) then
--						rState	<= stEnd;
--					else
--						rState	<= stMisc47;
--					end if;



				when stEnd			=>
					rState	<= stEnd;
					
				when others			=>
					rState	<= stIdle;
				
			end case;					
		end if;
	End Process u_rState;
	
	u_rUserBusy : Process (RstB, Clk) Is
	Begin
		if ( RstB='0' ) then
			rUserBusy	<= '1';
		elsif ( rising_edge(Clk) ) then
			rUserBusy	<= UserBusy;
		end if;
	End Process u_rUserBusy;
	
	u_rUserStart : Process (RstB, Clk) Is
	Begin
		if ( RstB='0' ) then
			rUserStart	<= '0';
		elsif ( rising_edge(Clk) ) then
			if not ( rState=stIdle or rState=stEnd ) then
				if ( rUserBusy='0' ) then
					rUserStart	<= '1';
				else
					rUserStart	<= '0';
				end if;
			else
				rUserStart	<= '0';
			end if;
		end if;
	End Process u_rUserStart;
	
	u_rUserRegAddr : Process (RstB, Clk) Is
	Begin
		if ( RstB='0' ) then
			rUserRegAddr	<= (others => '0');
		elsif ( rising_edge(Clk) ) then
			case rState is
				when stCmd1		=>	rUserRegAddr	<= CMD(23 downto 16);
				when stQpr		=>	rUserRegAddr	<= QPR(23 downto 16);
				
				when stBypass	=>	rUserRegAddr	<= BYPASS(23 downto 16);--***
				
				when stCmdSrst	=>	rUserRegAddr	<= CMD_SRST(23 downto 16);
				when stCmd2		=>	rUserRegAddr	<= CMD(23 downto 16);
				
				when stNconL	=> 	rUserRegAddr	<= NCON_L(23 downto 16);--*** 
				when stNconH	=>	rUserRegAddr	<= NCON_H(23 downto 16);--***
				when stNcodL	=>	rUserRegAddr	<= NCOD_L(23 downto 16);--***
				when stNcodH	=>	rUserRegAddr	<= NCOD_H(23 downto 16);--***
				
				when stPllM		=>	rUserRegAddr	<= PLL_M(23 downto 16);
				when stPllN		=>	rUserRegAddr	<= PLL_N(23 downto 16);
				when stPllP		=>	rUserRegAddr	<= PLL_P(23 downto 16);
				when stCmdLpll	=>	rUserRegAddr	<= CMD_LPLL(23 downto 16);
				when stCmd3		=>	rUserRegAddr	<= CMD(23 downto 16);
				when stMisc		=>	rUserRegAddr	<= MISC(23 downto 16);
				when stVidCtrl	=>	rUserRegAddr	<= VID_CTRL(23 downto 16);

				when stMisc47	=>	rUserRegAddr	<= MISC47(23 downto 16);

				when others		=>	rUserRegAddr	<= (others => '0');
			end case;
		end if;
	End Process u_rUserRegAddr;

	u_rUserData : Process (RstB, Clk) Is
	Begin
		if ( RstB='0' ) then
			rUserData	<= (others => '0');
		elsif ( rising_edge(Clk) ) then
			case rState is
				when stCmd1		=>	rUserData	<= CMD(15 downto 0 );
				when stQpr		=>	rUserData	<= QPR(15 downto 0 );
				
				when stBypass	=>	rUserData	<= BYPASS(15 downto 0 );--***
				
				when stCmdSrst	=>	rUserData	<= CMD_SRST(15 downto 0 );
				when stCmd2		=>	rUserData	<= CMD(15 downto 0 );

				when stNconL	=> 	rUserData	<= NCON_L(15 downto 0 );--*** 
				when stNconH	=>	rUserData	<= NCON_H(15 downto 0 );--***
				when stNcodL	=>	rUserData	<= NCOD_L(15 downto 0 );--***
				when stNcodH	=>	rUserData	<= NCOD_H(15 downto 0 );--***

				when stPllM		=>	rUserData	<= PLL_M(15 downto 0 );
				when stPllN		=>	rUserData	<= PLL_N(15 downto 0 );
				when stPllP		=>	rUserData	<= PLL_P(15 downto 0 );
				when stCmdLpll	=>	rUserData	<= CMD_LPLL(15 downto 0 );
				when stCmd3		=>	rUserData	<= CMD(15 downto 0 );
				when stMisc		=>	rUserData	<= MISC(15 downto 0 );
				when stVidCtrl	=>	rUserData	<= VID_CTRL(15 downto 0 );

				when stMisc47	=>	rUserData	<= MISC47(15 downto 0 );


				when others		=>	rUserData	<= (others => '0');		
			end case;
		end if;
	End Process u_rUserData;
	
	u_rConfigRdy : Process (RstB, Clk) Is
	Begin
		if ( RstB='0' ) then
			rConfigRdy	<= '0';
		elsif ( rising_edge(Clk) ) then
			if ( rState=stEnd ) then
				rConfigRdy	<= '1';
			else
				rConfigRdy	<= '0';
			end if;
		end if;
	End Process u_rConfigRdy;

End Architecture rtl;