----------------------------------------------------------------------------------

----------------------------------------------------------------------------------



-- Filename     CrtCtrl.vhd



-- Title        Data Comparator from Fifo16i32o



--



-- Company      Design Gateway Co., Ltd.



-- Project      JAMPII



-- PJ No.       RD0505



-- Syntax       VHDL



-- Note         



--



--



-- Version      1.00



-- Author       P.Kosol



-- Date         2008/09/11



-- Remark       New Creation



----------------------------------------------------------------------------------
----------------------------------------------------------------------------------

Library	IEEE;
Use		IEEE.STD_LOGIC_1164.all;
Use		IEEE.STD_LOGIC_UNSIGNED.all;

Entity CrtCtrl is
Generic
(
	MAXROW	: std_logic_vector(13 downto 0) := "00000000000100"--(N-1 rows)
);
Port(
		Clk100		: in	std_logic;
		Clk25		: in	std_logic;
		RstB		: in	std_logic;
		
		-- Command (SdrmCtrl)
		Busy		: in	std_logic;
		CmdReq		: out	std_logic;
		Cmd			: out	std_logic;
		RdAddr		: out	std_logic_vector(14 downto 0 );
		WrCnt		: in	std_logic_vector(10 downto 9 );
				
		-- Data (FIFO)
		AlmostEmpty	: in	std_logic;
		Empty		: in	std_logic;
		RdEn		: out	std_logic;
		RdData		: in	std_logic_vector(31 downto 0 );
		
		-- VGA
		Hsync_out	: out	std_logic;   
		Vsync_out	: out	std_logic;
		Blank_out	: out	std_logic;   
		Red_out		: out	std_logic_vector( 7 downto 0 );		
		Green_out	: out	std_logic_vector( 7 downto 0 );	
		Blue_out 	: out	std_logic_vector( 7 downto 0 );			
				
		-- Handshake with PtrnGen
		RdDataEn	: in	std_logic;
		PageSel		: in	std_logic
	);
End Entity CrtCtrl;

Architecture rtl of CrtCtrl is

--------------------------------------------------------------------------------
-- Constant Declaration
--------------------------------------------------------------------------------

	type state is(
					stIdle,
					stCmdRead,
					stReadData,
					stEnd
				);
				
	signal	rState		: state;
	
--------------------------------------------------------------------------------
-- Signal Declaration
--------------------------------------------------------------------------------

	signal	rCmdReq		: std_logic;
	signal	rRdAddr		: std_logic_vector(14 downto 0);
	signal	rRdEn		: std_logic;
	
Begin
	
--------------------------------------------------------------------------------
-- Output Assignment
--------------------------------------------------------------------------------
	CmdReq	<=	rCmdReq;
	Cmd		<=	'1';
	RdAddr	<=	rRdAddr;
	RdEn	<=	rRdEn;

--------------------------------------------------------------------------------
-- DFF
--------------------------------------------------------------------------------
	
	-- Command Path ------------------------------------------------------------
	u_rState : Process (RstB, Clk100) Is
	Begin
		if ( RstB='0' ) then
			rState	<= stIdle;
		elsif ( rising_edge(Clk100) ) then
			case (rState) is
			
				when stIdle		=>
					if ( Busy='0' and WrCnt(10)='0' and  RdDataEn='1') then
						rState	<= stCmdRead;
					else
						rState	<= stIdle;
					end if;
					
				when stCmdRead	=>
					if ( Busy='1' ) then
						rState	<= stReadData;
					else
						rState	<= stCmdRead;
					end if;
					
				when stReadData =>
					if ( Busy='0' ) then
						if ( rRdAddr(13 downto 0)=MAXROW ) then
							rState	<= stEnd;
						else
							rState	<= stIdle;
						end if;
					else
						rState	<= stReadData;
					end if;
					
				when stEnd		=>
					rState	<= stIdle;
				
				when others		=>
					rState	<= stIdle;
					
			end case;
		end if;
	End Process u_rState;
	
	u_rCmdReq : Process (RstB, Clk100) Is
	Begin
		if ( RstB='0' ) then
			rCmdReq	<= '0';
		elsif ( rising_edge(Clk100) ) then
			if ( Busy='1' ) then
				rCmdReq	<= '0';
			elsif ( rState=stCmdRead ) then
				rCmdReq	<= '1';
			else
				rCmdReq	<= rCmdReq;
			end if;
		end if;
	End Process u_rCmdReq;
	
	u_rRdAddr : Process (RstB, Clk100) Is
	Begin
		if ( RstB='0' ) then
			rRdAddr	<= (others => '0');
		elsif ( rising_edge(Clk100) ) then
			if ( rState=stReadData and Busy='0' ) then
				if ( rRdAddr(13 downto 0)=MAXROW ) then
					rRdAddr(13 downto 0)	<= (others => '0');
				else
					rRdAddr(13 downto 0)	<= rRdAddr(13 downto 0) + 1;
				end if;
			else
				rRdAddr(13 downto 0)	<= rRdAddr(13 downto 0);
			end if;
			
			if ( rState=stCmdRead ) then
				rRdAddr(14)		<= PageSel;
			else
				rRdAddr(14)		<= rRdAddr(14);
			end if;
		end if;
	End Process u_rRdAddr;
	
	-- Data Path ---------------------------------------------------------------
	u_rRdEn : Process (RstB, Clk25) Is
	Begin
		if ( RstB='0' ) then
			rRdEn	<= '0';
		elsif ( rising_edge(Clk25) ) then
			if ( AlmostEmpty='0' ) then
				rRdEn	<= '1';
			else
				rRdEn	<= '0';
			end if;
		end if;
	End Process u_rRdEn;
			
End Architecture rtl;