----------------------------------------------------------------------------------

----------------------------------------------------------------------------------


-- Filename     Master.vhd


-- Title        Master for i2c_master_byte_ctrl.vhd


--


-- Company      Design Gateway Co., Ltd.


-- Project      JAMPII


-- PJ No.       RD0505


-- Syntax       VHDL


-- Note         


--


--


-- Version      1.00


-- Author       P.Kosol


-- Date         2008/10/03


-- Remark       New Creation


----------------------------------------------------------------------------------

----------------------------------------------------------------------------------

Library	IEEE;
Use		IEEE.STD_LOGIC_1164.all;
Use		IEEE.STD_LOGIC_ARITH.all;

Entity Master is
Port(
		Clk    		: in 	std_logic;
		RstB 		: in 	std_logic;
				
		-- User Interface
		-- Input Signal
		UserStart	: in	std_logic;
		UserDevAddr	: in	std_logic_vector( 6 downto 0 );
		UserRegAddr	: in	std_logic_vector( 7 downto 0 );
		UserData	: in	std_logic_vector(15 downto 0 );
		-- Output Signal
		UserBusy	: out	std_logic;
		
		
		-- i2c_master_byte_ctrl Interface
		-- Aux signals
		ena    		: out 	std_logic;
		clk_cnt 	: out 	unsigned(15 downto 0);
		-- Output signals
		start		: out 	std_logic;
		read		: out 	std_logic;
		write		: out 	std_logic;
		stop		: out 	std_logic;
		ack_in 		: out	std_logic;
		din    		: out 	std_logic_vector( 7 downto 0);
		-- Input signals
		cmd_ack  	: in	std_logic	
	);
End Entity Master;

Architecture rtl of Master is
----------------------------------------------------------------------------------

-- Constant Declaration
----------------------------------------------------------------------------------
	type state is (
					stIdle,
					stGetData,
					stStart,
					stDevAddr,
					stRegAddr,
					stDataL,
					stDataH,
					stStop
				);
				
	signal	rState	: state;
----------------------------------------------------------------------------------

-- Signal Declaration
----------------------------------------------------------------------------------	
	signal	rUserBusy		: std_logic;
	signal	rUserDevAddr	: std_logic_vector( 6 downto 0 );
	signal	rUserRegAddr	: std_logic_vector( 7 downto 0 );
	signal	rUserDataL		: std_logic_vector( 7 downto 0 );
	signal	rUserDataH		: std_logic_vector(15 downto 8 );
	
	signal	rClkCnt			: unsigned(15 downto 0 );
	signal	rStart			: std_logic;
	signal	rWrite			: std_logic;
	signal	rStop			: std_logic;
	signal	rDin			: std_logic_vector( 7 downto 0 );
	
Begin
----------------------------------------------------------------------------------

-- Output Assignment
----------------------------------------------------------------------------------
	UserBusy	<= rUserBusy;
	ena			<= '1';
	clk_cnt		<= rClkCnt;
	start		<= rStart;
	read		<= '0';
	write		<= rWrite;
	stop		<= rStop;
	ack_in		<= '0';
	din			<= rDin;
	
----------------------------------------------------------------------------------

-- DFF
----------------------------------------------------------------------------------	
	u_rState : Process (RstB, Clk) Is
	Begin
		if ( RstB='0' ) then
			rState	<= stIdle;
		elsif ( rising_edge(Clk) ) then
			case rState is
			
				when stIdle		=>
					if ( UserStart='1' ) then
						rState	<= stGetData;
					else
						rState	<= stIdle;
					end if;
					
				when stGetData	=>
					rState	<= stStart;
				
				when stStart	=>
					rState	<= stDevAddr;

				when stDevAddr	=>
					if ( cmd_ack='1' ) then	
						rState	<= stRegAddr;
					else
						rState	<= stDevAddr;
					end if;
					
				when stRegAddr	=>
					if ( cmd_ack='1' ) then
						rState	<= stDataL;
					else
						rState	<= stRegAddr;
					end if;
					
				when stDataL	=>
					if ( cmd_ack='1' ) then
						rState	<= stDataH;
					else
						rState	<= stDataL;
					end if;
				
				when stDataH	=>
					if ( cmd_ack='1' ) then
						rState	<= stStop;
					else
						rState	<= stDataH;
					end if;
					
				when stStop		=>
					if ( cmd_ack='1' ) then
						rState	<= stIdle;
					else
						rState	<= stStop;
					end if;	
					
				when others		=>
					rState	<= stIdle;
					
			end case;								
		end if;
	End Process u_rState;
	
	-- User Interface ----------------------------------------------------------
	u_rUserBusy : Process (RstB, Clk) Is
	Begin
		if ( RstB='0' ) then
			rUserBusy	<= '1';
		elsif ( rising_edge(Clk) ) then
			if ( rState=stIdle ) then
				rUserBusy	<= '0';
			else
				rUserBusy	<= '1';
			end if;
		end if;
	End Process u_rUserBusy;
	
	u_rUserDevAddr : Process (RstB, Clk) Is
	Begin
		if ( RstB='0' ) then
			rUserDevAddr	<= (others => '0');
		elsif ( rising_edge(Clk) ) then
			if ( rState=stGetData ) then
				rUserDevAddr	<= UserDevAddr;
			else
				rUserDevAddr	<= rUserDevAddr;
			end if;
		end if;
	End Process u_rUserDevAddr;
	
	u_rUserRegAddr : Process (RstB, Clk) Is
	Begin
		if ( RstB='0' ) then
			rUserRegAddr	<= (others => '0');
		elsif ( rising_edge(Clk) ) then
			if ( rState=stGetData ) then
				rUserRegAddr	<= UserRegAddr;
			else
				rUserRegAddr	<= rUserRegAddr;
			end if;
		end if;
	End Process u_rUserRegAddr;
	
	u_rUserData : Process (RstB, Clk) Is
	Begin
		if ( RstB='0' ) then
			rUserDataL	<= (others => '0');
			rUserDataH	<= (others => '0');
		elsif ( rising_edge(Clk) ) then
			if ( rState=stGetData ) then
				rUserDataL	<= UserData( 7 downto 0 );
				rUserDataH	<= UserData(15 downto 8 );
			else
				rUserDataL	<= rUserDataL;
				rUserDataH	<= rUserDataH;
			end if;
		end if;
	End Process u_rUserData;
	
	
	-- ByteCtrl Interface -------------------------------------------------------	
	u_rClkCnt : Process (RstB, Clk) Is
	Begin
		if ( RstB='0' ) then
			rClkCnt	<= x"0031";-- 31 For 400 kHz SCL
		elsif ( rising_edge(Clk) ) then
			if ( rState=stGetData ) then
				rClkCnt	<= x"0031";
			else
				rClkCnt	<= rClkCnt;
			end if;
		end if;
	End Process u_rClkCnt;
	
	u_rDin : Process (RstB, Clk) Is
	Begin
		if ( RstB='0' ) then
			rDin	<=	(others => '0');
		elsif ( rising_edge(Clk) ) then
			case ( rState ) is
				when stDevAddr	=>
					rDin	<= rUserDevAddr & '0';
				when stRegAddr	=>
					rDin	<= rUserRegAddr;
				when stDataL	=>
					rDin	<= rUserDataL;
				when stDataH	=>
					rDin	<= rUserDataH;
				when others		=>
					rDin	<= (others => '0');
			end case;
		end if;
	End Process u_rDin;

	u_rStart : Process (RstB, Clk) Is
	Begin
		if ( RstB='0' ) then
			rStart	<= '0';
		elsif ( rising_edge(Clk) ) then
			if ( rState=stStart ) then
				rStart	<= '1';
			else
				rStart	<= '0';
			end if;
		end if;
	End Process u_rStart;

	u_rWrite : Process (RstB, Clk) Is
	Begin
		if ( RstB='0' ) then
			rWrite	<= '0';
		elsif ( rising_edge(Clk) ) then
			if ( rState=stStart ) then
				rWrite	<= '1';
			elsif ( rState=stDataH ) then
				rWrite	<= '0';
			else
				rWrite	<= rWrite;
			end if;
		end if;
	End Process u_rWrite;
	
	u_rStop : Process (RstB, Clk) Is
	Begin
		if ( RstB='0' ) then
			rStop	<= '0';
		elsif ( rising_edge(Clk) ) then
			if ( rState=stStop ) then
				if ( cmd_ack='1' ) then
					rStop	<= '0';
				else
					rStop	<= '1';
				end if;
			else
				rStop	<= '0';
			end if;
		end if;
	End Process u_rStop;

End Architecture rtl;