----------------------------------------------------------------------------------
----------------------------------------------------------------------------------


-- Filename     PtrnGen.vhd


-- Title        Pattern Generator to Fifo16i16o


--


-- Company      Design Gateway Co., Ltd.


-- Project      JAMPII


-- PJ No.       RD0505


-- Syntax       VHDL


-- Note         


--


--


-- Version      1.00


-- Author       P.Kosol


-- Date         2008/09/10


-- Remark       New Creation


----------------------------------------------------------------------------------
----------------------------------------------------------------------------------
Library	IEEE;
Use		IEEE.STD_LOGIC_1164.all;
Use		IEEE.STD_LOGIC_UNSIGNED.all;

Entity PtrnGen is
Generic (
		MAXROW		: std_logic_vector( 13 downto 0 ) := "00000000000100";--(N-1 rows)
		MAXLINE 	: std_logic_vector(  8 downto 0 ) := "000000001"--(N-1 lines)
	);
Port (
		Clk100		: in	std_logic;
		Clk12		: in	std_logic;
		RstB		: in	std_logic;
		
		-- Command (SdrmCtrl)
		Busy		: in	std_logic;
		CmdReq		: out	std_logic;
		Cmd			: out	std_logic;
		WrAddr		: out	std_logic_vector(14 downto 0);	
		RdCnt		: in	std_logic_vector(10 downto 9);
		
		-- Data (FIF0)
		WrCnt		: in	std_logic_vector( 9 downto 8);
		Full		: in	std_logic;
		WrEn		: out 	std_logic;
		WrData		: out 	std_logic_vector(31 downto 0);
		
		-- Handshake Signals
		ConfigRdy	: in	std_logic;
		RdDataEn	: out	std_logic;
		PageSel		: out	std_logic
	);
End Entity;		

Architecture rtl of PtrnGen is

-------------------------------------------------------------------------------
-- Constant Declaration
-------------------------------------------------------------------------------
	type State is(
					stIdle,
					stCmdWrite,
					stWriteData,
					stSwPage,
					stWait,
					stEnd
				);
	
	signal	rState			: State;

-------------------------------------------------------------------------------
-- Signal Declaration
-------------------------------------------------------------------------------
	signal	rCmdReq			: std_logic;
	signal	rWrAddr			: std_logic_vector(14 downto 0 );
	signal	rWrEn			: std_logic;
	signal	rRdDataEn		: std_logic;
	signal	rFull			: std_logic_vector( 7 downto 0 );
	signal	rCnt3			: std_logic_vector( 1 downto 0 );
	
	signal	rBarWidth		: std_logic_vector( 6 downto 0 );
	signal	rBarCnt			: std_logic_vector( 2 downto 0 );
	signal	rPattData		: std_logic_vector(23 downto 0 );
	signal	rLineCnt		: std_logic_vector( 8 downto 0 );

	signal	rLineWidth		: std_logic_vector( 5 downto 0 );
	signal	rPattLineCnt	: std_logic_vector( 2 downto 0 );
	signal	rDelayCnt		: std_logic_vector(30 downto 0 );
	signal	rCounter		: std_logic_vector(23 downto 0 );
	
Begin

-------------------------------------------------------------------------------
-- Output Assignment
-------------------------------------------------------------------------------
	CmdReq		<=	rCmdReq;
	Cmd			<=	'0';
	WrAddr		<=	rWrAddr;
	WrEn		<=	rWrEn;							
	WrData		<=	x"00" & rPattData;
	RdDataEn	<=	rRdDataEn;
	PageSel		<=	not rWrAddr(14);

-------------------------------------------------------------------------------
-- DFF
-------------------------------------------------------------------------------

	-- Command Path -----------------------------------------------------------
	u_rState : Process (RstB, Clk100) Is
	Begin
		if ( RstB='0' ) then
			rState	<= stIdle;
		elsif ( rising_edge(Clk100) ) then
			case (rState) is
				
				when stIdle		=>
					if ( Busy='0' and not(RdCnt(10 downto 9)="00") ) then
						rState	<= stCmdWrite;
					else
						rState	<= stIdle;
					end if;
					
				when stCmdWrite	=>
					if ( Busy='1' ) then
						rState	<= stWriteData;
					else
						rState	<= stCmdWrite;
					end if;
				
				when stWriteData	=>
					if ( Busy='0') then
						if ( rWrAddr(13 downto 0)=MAXROW ) then
							rState	<= stSwPage;
						else
							rState	<= stIdle;
						end if;
					else
						rState	<= stWriteData;
					end if;

				when stSwPage		=>
					rState		<= stWait;
				
				when stWait		=>
					if ( rDelayCnt(28)='1' ) then
						rState	<= stEnd;
					else
						rState	<= stWait;
					end if;
					
				when stEnd		=>
					rState	<= stIdle;			
				
				when others		=>
					rState	<= stIdle;
										
			end case;
		end if;
	End Process u_rState;
	
	u_rCmdReq : Process (RstB, Clk100) Is
	Begin
		if ( RstB ='0' ) then
			rCmdReq	<= '0';
		elsif ( rising_edge(Clk100) ) then
			if ( Busy='1' ) then
				rCmdReq		<= '0';
			elsif ( rState=stCmdWrite ) then
				rCmdReq		<= '1';
			else
				rCmdReq		<= rCmdReq;
			end if;
		end if;
	End Process u_rCmdReq;

	u_rWrAddr : Process (RstB, Clk100) Is
	Begin
		if ( RstB='0' ) then
			rWrAddr	<= (others => '0');
		elsif ( rising_edge(Clk100) ) then
			if ( rState=stWriteData and Busy='0' ) then
				if ( rWrAddr(13 downto 0)=MAXROW ) then
					rWrAddr(13 downto 0) 	<= (others=>'0');
				else
					rWrAddr(13 downto 0)	<= rWrAddr(13 downto 0) + 1;
				end if;
			else
				rWrAddr(13 downto 0)	<= rWrAddr(13 downto 0);
			end if;
			
			if ( rState=stSwPage ) then
				rWrAddr(14)		<= not rWrAddr(14);
			else
				rWrAddr(14)		<= rWrAddr(14);
			end if;

		end if;
	End Process u_rWrAddr;

	-- Handshake signals --------------------------------------------------------
	u_rRdDataEn : Process (RstB, Clk100) Is
	Begin
		if ( RstB='0' ) then
			rRdDataEn	<= '0';
		elsif ( rising_edge(Clk100) ) then
			if ( rState=stEnd ) then
				rRdDataEn	<= '1';
			else
				rRdDataEn	<= rRdDataEn;
			end if;
		end if;
	End Process u_rRdDataEn;
	
	u_rDelayCnt : Process (RstB, Clk100) Is
	Begin
		if ( RstB='0' ) then
			rDelayCnt	<= (others => '0');
		elsif ( rising_edge(Clk100) ) then
			if ( rState=stWait ) then
				rDelayCnt	<= rDelayCnt + 1;
			else
				rDelayCnt	<= (others => '0');
			end if;
		end if;
	End Process u_rDelayCnt;		

	-- Data Path --------------------------------------------------------------
	u_rFull : Process (RstB, Clk12) Is
	Begin
		if ( RstB='0' ) then
			rFull	<= (others=>'1');
		elsif ( rising_edge(Clk12) ) then
			rFull	<= rFull(6 downto 0) & Full;			
		end if;
	End Process u_rFull;
	
	u_rWrEn : Process (RstB, Clk12) Is
	Begin
		if ( RstB='0' ) then
			rWrEn	<= '0';
		elsif ( rising_edge(Clk12) ) then
			if ( ConfigRdy='1' ) then
				if ( WrCnt(9 downto 8)="11" or rFull(7)='1' ) then
					rWrEn	<= '0';
				else
					rWrEn	<= '1';
				end if;
			else
				rWrEn	<= '0';
			end if;
		end if;
	End Process u_rWrEn;

	u_rCounter : Process (RstB, Clk12) Is
	Begin
		if ( RstB='0' ) then
			rCounter		<= (others=>'0');
		elsif ( rising_edge(Clk12) ) then
			if ( ConfigRdy='1' ) then
				if ( WrCnt(9 downto 8)="11" or rFull(7)='1' ) then
					rCounter	<= rCounter;
				else
					rCounter	<= rCounter + 1;
				end if;
			else
				rCounter	<= rCounter;
			end if;
		end if;
	End Process u_rCounter;
	
	u_rBarWidth : Process (RstB, Clk12) Is
	Begin
		if ( RstB='0' ) then
			rBarWidth	<= (others => '0');
		elsif ( rising_edge(Clk12) ) then
			if ( ConfigRdy='1' ) then
				if ( WrCnt(9 downto 8)="11" or rFull(7)='1' ) then
					rBarWidth	<= rBarWidth;
				else
					if ( rBarWidth="1001111" ) then
						rBarWidth	<= (others=>'0');
					else
						rBarWidth	<= rBarWidth + 1;
					end if;
				end if;
			else
				rBarWidth	<= rBarWidth;
			end if;
		end if;
	End Process u_rBarWidth;
	
	u_BarCnt : Process (RstB, Clk12) Is
	Begin
		if ( RstB='0' ) then
			rBarCnt	<= "000";
		elsif ( rising_edge(Clk12) ) then
			if ( rBarWidth="1001111" and rWrEn='1' ) then
				rBarCnt	<= rBarCnt + 1;
			else
				rBarCnt	<= rBarCnt;
			end if;
		end if;
	End Process u_BarCnt;

	u_rLineWidth : Process (RstB, Clk12) Is
	Begin
		if ( RstB='0' ) then
			rLineWidth		<= (others=>'0');
		elsif ( rising_edge(Clk12) ) then
			if ( rBarWidth="1001111" and rBarCnt="111" and rWrEn='1' ) then
				if ( rLineWidth="111011" ) then
					rLineWidth	<= (others=>'0');
				else
					rLineWidth	<= rLineWidth + 1;
				end if;
			else
				rLineWidth		<= rLineWidth;
			end if;
		end if;
	End Process u_rLineWidth;
	
	u_rPattLineCnt : Process (RstB, Clk12) Is
	Begin
		if ( RstB='0' ) then
			rPattLineCnt	<= "000";
		elsif ( rising_edge(Clk12) ) then
			if ( rBarWidth="1001111" and rBarCnt="111" and rWrEn='1' ) then
				if ( rLineWidth="111011" ) then
					rPattLineCnt	<= rPattLineCnt + 1;
				else
					rPattLineCnt	<= rPattLineCnt;
				end if;
			else
				rPattLineCnt		<= rPattLineCnt;
			end if;			
		end if;
	End Process u_rPattLineCnt;
	
	u_rLineCnt : Process (RstB, Clk12) Is
	Begin
		if ( RstB='0' ) then
			rLineCnt	<= (others => '0');
		elsif ( rising_edge(Clk12) ) then
			if ( rBarWidth="1001111" and rBarCnt="111" and rWrEn='1' ) then
				if ( rLineCnt=MAXLINE ) then
					rLineCnt	<= (others => '0') ;
				else
					rLineCnt	<= rLineCnt + 1;
				end if;
			else
				rLineCnt	<= rLineCnt;
			end if;
		end if;
	End Process u_rLineCnt;
	
	u_rCnt3 : Process (RstB, Clk12) Is
	Begin
		if ( RstB='0' ) then
			rCnt3	<= "00";
		elsif ( rising_edge(Clk12) ) then
			if ( rBarWidth="1001111" and rBarCnt="111" and rLineCnt=MAXLINE ) then
				if ( rCnt3="10" ) then
					rCnt3	<= "00";
				else
					rCnt3	<= rCnt3 + 1;
				end if;
			else
				rCnt3	<= rCnt3;
			end if;
		end if;
	End Process u_rCnt3;

	u_rPattData : Process (RstB, Clk12) Is
	Begin
		if ( RstB='0' ) then
			rPattData		<= x"000000";
		elsif ( rising_edge(Clk12) ) then
			if ( rCnt3="00" ) then
				case (rBarCnt) is
					when "000"	=> rPattData <= x"FFFFFF";
					when "001"	=> rPattData <= x"FFFF00";
					when "010"	=> rPattData <= x"FF00FF";
					when "011"	=> rPattData <= x"FF0000";
					when "100"	=> rPattData <= x"00FFFF";
					when "101"	=> rPattData <= x"00FF00";
					when "110"	=> rPattData <= x"0000FF";
					when others	=> rPattData <= x"000000";
				end case;				
			elsif ( rCnt3="01" ) then
				case (rPattLineCnt) is				
					when "000"	=> rPattData <= x"000000";
					when "001"	=> rPattData <= x"0000FF";
					when "010"	=> rPattData <= x"00FF00";
					when "011"	=> rPattData <= x"00FFFF";
					when "100"	=> rPattData <= x"FF0000";
					when "101"	=> rPattData <= x"FF00FF";
					when "110"	=> rPattData <= x"FFFF00";
					when others	=> rPattData <= x"FFFFFF";
				end case;
			else
				case (rBarCnt) is
					when "000"	=> rPattData <= x"FF0000";
					when "001"	=> rPattData <= x"FF0000";
					when "010"	=> rPattData <= x"00FF00";
					when "011"	=> rPattData <= x"00FF00";
					when "100"	=> rPattData <= x"0000FF";
					when "101"	=> rPattData <= x"0000FF";
					when "110"	=> rPattData <= x"FFFFFF";
					when others	=> rPattData <= x"FFFFFF";
				end case;
			end if;			
		end if;
	End Process u_rPattData;
	
End Architecture rtl;