----------------------------------------------------------------------------------
----------------------------------------------------------------------------------


-- Filename     SdrmCtrl.vhd


-- Title        SDRAM Controller


--


-- Company      Design Gateway Co., Ltd.


-- Project      JAMPII


-- PJ No.       RD0505


-- Syntax       VHDL


-- Note         


--


--


-- Version      1.00


-- Author       P.Kosol


-- Date         2008/09/08


-- Remark       New Creation


----------------------------------------------------------------------------------

----------------------------------------------------------------------------------
Library IEEE;
USE		IEEE.STD_LOGIC_1164.all;
USE		IEEE.STD_LOGIC_UNSIGNED.all;
USE		IEEE.NUMERIC_STD.all;

Entity	SdrmCtrl is
Port(
		Clk					: in	std_logic;
		RstB				: in	std_logic;

		-- User Interface
		UserCmd				: in	std_logic;-- '0': Write, '1': Read
		UserCmdReq			: in	std_logic;
		UserAddr			: in	std_logic_vector(14 downto 0 );
		UserBusy			: out	std_logic;
		UserDataReq			: out	std_logic;
		UserDataOut			: in	std_logic_vector(15 downto 0 );
		UserDataValid		: out	std_logic;
		UserDataIn			: out	std_logic_vector(15 downto 0);

		-- SDRAM Interface
		sd_data				: inout std_logic_vector(15 downto 0);
		sd_add				: out	std_logic_vector(12 downto 0);
		sd_ras 				: out 	std_logic;
		sd_cas 				: out 	std_logic;
		sd_we 				: out 	std_logic;
		sd_ba				: out	std_logic_vector( 1 downto 0);
		Clk_Sdram 			: out 	std_logic;
		sd_cke 				: out 	std_logic;
		sd_cs				: out	std_logic;
		sd_dqm				: out	std_logic_vector( 1 downto 0)
	);
End Entity SdrmCtrl;

Architecture rtl of SdrmCtrl is

----------------------------------------------------------------------------------	
-- Constant Declaration
----------------------------------------------------------------------------------
				  
----------------------------------------------------------------------------------	
-- Component Declaration
----------------------------------------------------------------------------------

	Component Sdrm
	port(
		Reset 				: in 	std_logic;
		Clkp 				: in 	std_logic;

		-- Internal interface
		we_rn 				: in 	std_logic;
		data_addr_n 		: in 	std_logic;
		AddrIn				: in	std_logic_vector(31 downto 0);
		DataIn				: in	std_logic_vector(15 downto 0);
		DataOut				: out	std_logic_vector(15 downto 0);

		-- Sdram interface
		sd_data				: inout std_logic_vector(15 downto 0);
		sd_add				: out	std_logic_vector(12 downto 0);
		sd_ras 				: out 	std_logic;
		sd_cas 				: out 	std_logic;
		sd_we 				: out 	std_logic;
		sd_ba				: out	std_logic_vector(1 downto 0);
		Clk_Sdram 			: out 	std_logic;
		sd_cke 				: out 	std_logic;
		sd_cs				: out	std_logic;
		sd_dqm				: out	std_logic_vector(1 downto 0)
	);
	End Component;

	Component UserIf
	port(
		Clk 				: in 	std_logic;
		RstB 				: in 	std_logic;

		-- User Interface
		UserCmd				: in	std_logic;	-- '0': Write, '1': Read
		UserCmdReq			: in	std_logic;
		UserAddr			: in	std_logic_vector(14 downto 0);
		UserBusy			: out	std_logic;
		UserDataReq			: out	std_logic;
		UserDataOut			: in	std_logic_vector(15 downto 0);
		UserDataValid		: out	std_logic;
		UserDataIn			: out	std_logic_vector(15 downto 0);

		-- Internal interface
		WeRn 				: out 	std_logic;
		DataAddrN	 		: out 	std_logic;
		AddrIn				: out	std_logic_vector(31 downto 0);
		DataIn				: out	std_logic_vector(15 downto 0);
		DataOut				: in	std_logic_vector(15 downto 0)
	);
	End Component;

----------------------------------------------------------------------------------	
-- Signal Declaration
----------------------------------------------------------------------------------

	signal	Reset			: std_logic;
	
	-- Internal Interface
	signal	we_rn 	   		: std_logic;	
	signal	data_addr_n		: std_logic; 
	signal	AddrIn	   		: std_logic_vector(31 downto 0);	
	signal	DataIn	   		: std_logic_vector(15 downto 0);	
	signal	DataOut	   		: std_logic_vector(15 downto 0);

Begin
	
----------------------------------------------------------------------------------	
-- Output Assignment
----------------------------------------------------------------------------------

	Reset		<= not RstB;

----------------------------------------------------------------------------------	
-- Component Mapping
----------------------------------------------------------------------------------

	u_Sdrm : Sdrm
	port map(
		Reset			=>	Reset			, 					
		Clkp 			=>	Clk 			,		

		-- Internal interface 
		we_rn 			=>	we_rn			,			
		data_addr_n 	=>	data_addr_n 	,		
		AddrIn			=>	AddrIn			,			
		DataIn			=>	DataIn			,			
		DataOut			=>	DataOut			,			

		-- Sdram interface
		sd_data			=>  sd_data			,			
		sd_add			=>  sd_add			,			
		sd_ras 			=>  sd_ras 			,			
		sd_cas 			=>  sd_cas 			,			
		sd_we 			=>  sd_we 			,			
		sd_ba			=>  sd_ba			,		
		Clk_Sdram 		=>  Clk_Sdram 		,			
		sd_cke 			=>  sd_cke 			,			
		sd_cs			=>  sd_cs			,		
		sd_dqm			=>  sd_dqm						
	);

	u_UserIf : UserIf
	port map(
		Clk 			=>  Clk 			,
		RstB 			=>  RstB 			,

		-- User Interface
		UserCmd			=>	UserCmd			,			
		UserCmdReq		=>	UserCmdReq		,	
		UserAddr		=>	UserAddr		,			
		UserBusy		=>	UserBusy		,		
		UserDataReq		=>	UserDataReq 	,
		UserDataOut		=>	UserDataOut		,
		UserDataValid	=>	UserDataValid	,
		UserDataIn		=>	UserDataIn		,	

		-- Internal Interface							    
		WeRn 			=>	we_rn	    	,
		DataAddrN		=>	data_addr_n		,
		AddrIn			=>	AddrIn	    	,	
		DataIn			=>	DataIn	    	,	
		DataOut			=>	DataOut	   	
	);								   

----------------------------------------------------------------------------------	
-- DFF
----------------------------------------------------------------------------------	

End Architecture rtl;