----------------------------------------------------------------------------------

----------------------------------------------------------------------------------



-- Filename     System2.vhd



-- Title        System2



--



-- Company      Design Gateway Co., Ltd.



-- Project      JAMPII



-- PJ No.       RD0505



-- Syntax       VHDL



-- Note         



--



--



-- Version      1.00



-- Author       P.Kosol



-- Date         2008/10/07



-- Remark       New Creation



----------------------------------------------------------------------------------
----------------------------------------------------------------------------------
Library	IEEE;
Use		IEEE.STD_LOGIC_1164.all;
Use		IEEE.STD_LOGIC_UNSIGNED.all;
Library UNISIM;
Use		UNISIM.vcomponents.all;

Entity System2 is
Port(
		SysClk			: in	std_logic;
		RstB			: in	std_logic;

		sd_data			: inout std_logic_vector(15 downto 0);
		sd_add			: out	std_logic_vector(12 downto 0);
		sd_ras 			: out 	std_logic;
		sd_cas 			: out 	std_logic;
		sd_we 			: out 	std_logic;
		sd_ba			: out	std_logic_vector( 1 downto 0);
		Clk_Sdram 		: out 	std_logic;
		sd_cke 			: out 	std_logic;
		sd_cs			: out	std_logic;
		sd_dqm			: out	std_logic_vector( 1 downto 0);
		
		scl				: inout	std_logic;
		sda				: inout	std_logic;		

		VGA_Clk			: in	std_logic;
		VGA_nRst		: out	std_logic;
		VGA_PixClk		: out	std_logic;
		VGA_nPixClk		: out	std_logic;
		VGA_Hsync		: out	std_logic;
		VGA_Hsync_out	: out	std_logic;   
		VGA_Vsync		: out 	std_logic;
		VGA_Vsync_out	: out	std_logic;
		VGA_Blank		: out 	std_logic;   
		VGA_Red			: out	std_logic_vector( 7 downto 0 );		
		VGA_Green		: out	std_logic_vector( 7 downto 0 );	
		VGA_Blue		: out	std_logic_vector( 7 downto 0 )
	);
End Entity System2;

Architecture rtl of System2 is

-------------------------------------------------------------------
-- Constant Declaration
-------------------------------------------------------------------
	-- 5 Rows = 2 Lines										
	constant	Hres		: std_logic_vector(10 downto 0 ):= "01100100000";-- Horizontal resolution (800 pixels/line)
	constant	Vres		: std_logic_vector(10 downto 0 ):= "01000001101";-- Vertical resolution (525 lines/frame)
	constant	HsyncWidth	: std_logic_vector( 6 downto 0 ):= "1100000";	 -- Horizontal sync width (96 pixels)
	constant	VsyncWidth	: std_logic_vector( 6 downto 0 ):= "0000010";	 -- Vertical sync width (2 lines)
	constant	Hbackporch	: std_logic_vector( 6 downto 0 ):= "0101101";	 -- Horizontal back porch 45 pixels)
	constant	Hactive		: std_logic_vector(10 downto 0 ):= "01010000000";-- Horizontal active pixel (640 pixels/line)
	constant	Vbackporch	: std_logic_vector( 6 downto 0 ):= "0011110";	 -- Vertical back porch (30 lines)
	constant	Vactive		: std_logic_vector(10 downto 0 ):= "00111100000";-- Vertical active line (480 lines/frame)
	constant	HVpolar		: std_logic_vector( 2 downto 0 ):= "110";		 -- Hsync/Vsync/Blank polarity (Sync Active Low)
	constant	MAXROW		: std_logic_vector(13 downto 0 ):= "00010010101111";-- 1200-1 rows
	constant	MAXLINE		: std_logic_vector( 8 downto 0 ):= "111011111";  -- 480-1 lines	

-------------------------------------------------------------------
-- Component Declaration
-------------------------------------------------------------------
	Component SdrmCtrl is
		Port (
			Clk				: in	std_logic;
			RstB			: in	std_logic;

			UserCmd			: in	std_logic;-- '0': Write, '1': Read
			UserCmdReq		: in	std_logic;
			UserAddr		: in	std_logic_vector(14 downto 0 );
			UserBusy		: out	std_logic;
			UserDataReq		: out	std_logic;
			UserDataOut		: in	std_logic_vector(15 downto 0 );
			UserDataValid	: out	std_logic;
			UserDataIn		: out	std_logic_vector(15 downto 0 );

			sd_data			: inout std_logic_vector(15 downto 0 );
			sd_add			: out	std_logic_vector(12 downto 0 );
			sd_ras 			: out 	std_logic;
			sd_cas 			: out 	std_logic;
			sd_we 			: out 	std_logic;
			sd_ba			: out	std_logic_vector( 1 downto 0 );
			Clk_Sdram 		: out 	std_logic;
			sd_cke 			: out 	std_logic;
			sd_cs			: out	std_logic;
			sd_dqm			: out	std_logic_vector( 1 downto 0 )
		);
	End Component SdrmCtrl;

	Component PtrnGen is
		Generic (
			MAXROW			: std_logic_vector( 13 downto 0 );
			MAXLINE 		: std_logic_vector(  8 downto 0 ) 
		);
		Port (
			Clk100			: in	std_logic;
			Clk12			: in	std_logic;
			RstB			: in	std_logic;

			Busy			: in	std_logic;
			CmdReq			: out	std_logic;
			Cmd				: out	std_logic;
			WrAddr			: out	std_logic_vector(14 downto 0 );	
			RdCnt			: in	std_logic_vector(10 downto 9 );
			
			WrCnt			: in	std_logic_vector( 9 downto 8 );
			Full			: in	std_logic;
			WrEn			: out 	std_logic;
			WrData			: out 	std_logic_vector(31 downto 0 );
			
			ConfigRdy		: in	std_logic;
			
			RdDataEn		: out	std_logic;
			PageSel			: out	std_logic
		);
	End Component PtrnGen;
	
	Component Fifo32i16o is
		Port (
			rst				: in 	std_logic;

			wr_clk			: in 	std_logic;
			wr_en			: in 	std_logic;
			din				: in 	std_logic_vector(31 downto 0 );
			wr_data_count	: out 	std_logic_vector( 9 downto 0 );
			full			: out 	std_logic;
						
			rd_clk			: in 	std_logic;
			rd_en			: in 	std_logic;
			dout			: out 	std_logic_vector(15 downto 0 );
			rd_data_count	: out 	std_logic_vector(10 downto 0 );
			empty			: out 	std_logic
		);
	End Component Fifo32i16o;

	Component HYSyncGen is
		Generic (
			Hres			: std_logic_vector(10 downto 0 );
			Vres			: std_logic_vector(10 downto 0 );
			HsyncWidth		: std_logic_vector( 6 downto 0 );	 
			VsyncWidth		: std_logic_vector( 6 downto 0 );	 
			Hbackporch		: std_logic_vector( 6 downto 0 );	 
			Hactive			: std_logic_vector(10 downto 0 );
			Vbackporch		: std_logic_vector( 6 downto 0 );	 
			Vactive			: std_logic_vector(10 downto 0 );
			HVpolar			: std_logic_vector( 2 downto 0 );		 

			MAXROW			: std_logic_vector(13 downto 0 );
			MAXLINE 		: std_logic_vector( 8 downto 0 )
		);
		Port (
			Clk100      	: in    std_logic;                      
			Clk25			: in	std_logic;
			RstB        	: in    std_logic;                      
	
			Busy			: in	std_logic;
			CmdReq			: out 	std_logic;
			Cmd				: out	std_logic;
			RdAddr			: out	std_logic_vector( 14 downto 0 );
			WrCnt			: in	std_logic_vector( 10 downto 9 );

			AlmostEmpty		: in	std_logic;
			Empty			: in	std_logic;
			RdEn			: out	std_logic;
			RdData			: in	std_logic_vector( 31 downto 0 );
		
			PageSel			: in	std_logic;
		
			Hsync_out   	: out   std_logic;                      
			Vsync_out   	: out   std_logic;                      
			Blank_out   	: out   std_logic;                      
			Red_out			: out	std_logic_vector( 7 downto 0 );	
			Green_out		: out	std_logic_vector( 7 downto 0 );	
			Blue_out		: out	std_logic_vector( 7 downto 0 )	
		);
	End Component HYSyncGen;
	
	Component Fifo16i32o is
		Port (
			rst				: in 	std_logic;

			wr_clk			: in 	std_logic;
			wr_en			: in 	std_logic;
			din				: in 	std_logic_vector(15 downto 0 );
			wr_data_count	: out 	std_logic_vector(10 downto 0 );
			full			: out 	std_logic;

			rd_clk			: in 	std_logic;
			rd_en			: in 	std_logic;
			dout			: out 	std_logic_vector(31 downto 0 );
			rd_data_count	: out 	std_logic_vector( 9 downto 0 );
			almost_empty	: out 	std_logic;
			empty			: out 	std_logic
	);
	End Component Fifo16i32o;

	Component Arbiter is
		Port (
			Clk				: in	std_logic;
			RstB			: in	std_logic;
			
			CmdReq			: out	std_logic;
			Cmd				: out	std_logic;
			Busy			: in	std_logic;
			Addr			: out	std_logic_vector(14 downto 0 );
			
			CmdReq1			: in	std_logic;
			Cmd1			: in	std_logic;
			Busy1			: out	std_logic;
			Addr1			: in	std_logic_vector(14 downto 0 );
			
			CmdReq2			: in	std_logic;
			Cmd2			: in	std_logic;
			Busy2			: out	std_logic;
			Addr2			: in	std_logic_vector(14 downto 0 )
	);
	End Component Arbiter;
	
	Component VideoConfig is
		Port (
			Clk    			: in 	std_logic;
			RstB 			: in 	std_logic;
			
			ConfigRdy		: out	std_logic;
			
			scl				: inout	std_logic;
			sda				: inout	std_logic	
	);
	End Component VideoConfig;

-------------------------------------------------------------------
-- Signal Declaration
-------------------------------------------------------------------
	signal	Clk100			: std_logic;
	signal	Clk25			: std_logic;
	signal	Clk12			: std_logic;
	signal	Clk6			: std_logic;
	signal	rClk			: std_logic_vector( 2 downto 0 );
	signal	PixClk			: std_logic;
	
	signal	UserCmd	   		: std_logic;-- '0': Write, '1':Read
	signal	UserCmdReq 		: std_logic;
	signal	UserAddr   		: std_logic_vector(14 downto 0 );
	signal	UserLength 		: std_logic_vector( 3 downto 0 );
	signal	UserBusy   		: std_logic;
	signal	UserDataReq		: std_logic;
	signal	UserDataOut		: std_logic_vector(15 downto 0 );
	signal	UserDataValid	: std_logic;
	signal	UserDataIn		: std_logic_vector(15 downto 0 );            		
	
	signal	CmdReq1			: std_logic;
	signal	Cmd1			: std_logic;
	signal	Busy1			: std_logic;
	signal	Addr1			: std_logic_vector(14 downto 0 );
	signal	WrEn			: std_logic;
	signal	WrData			: std_logic_vector(31 downto 0 );
	signal	WrCnt1			: std_logic_vector( 9 downto 0 );
	signal	RdCnt1			: std_logic_vector(10 downto 0 );
	signal	Full			: std_logic;
	
	signal	CmdReq2			: std_logic;
	signal	Cmd2			: std_logic;
	signal	Busy2			: std_logic;
	signal	Addr2			: std_logic_vector(14 downto 0 );
	signal	RdEn			: std_logic;
	signal	RdData			: std_logic_vector(31 downto 0 );
	signal	WrCnt2			: std_logic_vector(10 downto 0 );
	signal	AlmostEmpty		: std_logic;
	signal	Empty			: std_logic;
	
	signal	ConfigRdy		: std_logic;
	signal	wVGA_Hsync		: std_logic;   
	signal	wVGA_Vsync		: std_logic;
	signal	wVGA_Blank		: std_logic;   
	signal	wVGA_Red		: std_logic_vector( 7 downto 0 );		
	signal	wVGA_Green		: std_logic_vector( 7 downto 0 );	
	signal	wVGA_Blue 		: std_logic_vector( 7 downto 0 );	

	signal	RdDataEn		: std_logic;
	signal	PageSel			: std_logic;
	
Begin
	
-------------------------------------------------------------------
-- Output Assignment
-------------------------------------------------------------------
	VGA_nRst		<= '1';
	VGA_Hsync		<= wVGA_Hsync;
	VGA_Vsync		<= wVGA_Vsync;
	VGA_Blank		<= wVGA_Blank;
	VGA_Red			<= wVGA_Red;		
	VGA_Green		<= wVGA_Green;
	VGA_Blue		<= wVGA_Blue;
	VGA_Hsync_out	<= wVGA_Hsync;
	VGA_Vsync_out	<= wVGA_Vsync;
	
-------------------------------------------------------------------
-- Component Mapping
-------------------------------------------------------------------
	u_Clk100 : BUFG Port map( I => SysClk,	O => Clk100 );
	u_Clk25  : BUFG Port map( I => VGA_Clk, O => Clk25 );

	u_rClk : Process (RstB, Clk100) Is
	Begin
		if ( RstB='0' ) then
			rClk	<= (others => '0');
		elsif ( rising_edge(Clk100) ) then
			rClk	<= rClk + 1;
		end if;
	End Process u_rClk;
	
	u_Clk12	 : BUFG Port map( I => rClk(2), O => Clk12 );

	PixClk		<= not Clk25;
		
	u_PixClk : OBUFDS port map ( O => VGA_PixClk, OB => VGA_nPixClk, I => PixClk );

	u_SdrmCtrl : SdrmCtrl
	Port map (
		Clk				=> Clk100				,	
		RstB			=> RstB					,	

		UserCmd	   		=> UserCmd	   			,	
		UserCmdReq 		=> UserCmdReq 			,
		UserAddr   		=> UserAddr   			,
		UserBusy   		=> UserBusy   			,
		UserDataReq		=> UserDataReq			,
		UserDataOut		=> UserDataOut			,
		UserDataValid	=> UserDataValid		,
		UserDataIn		=> UserDataIn			,

		sd_data			=> sd_data				,
		sd_add			=> sd_add				,
		sd_ras 			=> sd_ras 				,
		sd_cas 			=> sd_cas 				,
		sd_we 			=> sd_we 				,
		sd_ba			=> sd_ba				,
		Clk_Sdram 		=> Clk_Sdram 			,
		sd_cke 			=> sd_cke 				,
		sd_cs			=> sd_cs				,
		sd_dqm			=> sd_dqm		
	);
		
	u_PtrnGen : PtrnGen
	Generic map	(
		MAXROW			=> MAXROW				,
		MAXLINE			=> MAXLINE
	)										   	
	Port map (								   	
		Clk100			=> Clk100  			   	,
		Clk12			=> Clk12			   	,
		RstB		  	=> RstB				   	,
											   	
		Busy		  	=> Busy1			   	,
		RdCnt		  	=> RdCnt1(10 downto 9) 	,
		CmdReq	  		=> CmdReq1			   	,
		Cmd		  		=> Cmd1				   	,
		WrAddr	  		=> Addr1			   	,
											   	
		WrCnt		  	=> WrCnt1( 9 downto 8) 	,
		Full		  	=> Full				   	,
		WrEn		  	=> WrEn				   	,
		WrData	  		=> WrData			   	,
											   	
		ConfigRdy		=> ConfigRdy			,
		
		RdDataEn	  	=> RdDataEn			   	,
		PageSel	  		=> PageSel											   
	);				   						   
					   						   
	u_Fifo32i16o : Fifo32i16o
	Port map (
		rst				=> RstB 			   	,
		
		wr_clk			=> Clk12			   	,
		wr_en			=> WrEn				   	,
		din				=> WrData			   	,
		wr_data_count	=> WrCnt1			   	,
		full			=> full				   	,
		
		rd_clk			=> Clk100			   	,
		rd_en			=> UserDataReq		   	,
		dout			=> UserDataOut		   	,
		rd_data_count	=> RdCnt1			   	,
		empty			=> open
	);

	u_HYSyncGen : HYSyncGen
	Generic map	(
		Hres			=> Hres					,
		Vres			=> Vres					,
		HsyncWidth		=> HsyncWidth			,
		VsyncWidth		=> VsyncWidth			,
		Hbackporch		=> Hbackporch			,
		Hactive			=> Hactive				,
		Vbackporch		=> Vbackporch			,
		Vactive			=> Vactive				,
		HVpolar			=> HVpolar				,
		MAXROW			=> MAXROW				,				
		MAXLINE			=> MAXLINE
	)											
	Port map (									
		Clk100      	=> Clk100     			,
		Clk25			=> Clk25				,
		RstB			=> RdDataEn				,
														
		Busy			=> Busy2				,
		CmdReq			=> CmdReq2				,
		Cmd				=> Cmd2					,
		RdAddr			=> Addr2				,
		WrCnt			=> WrCnt2(10 downto 9)	,	
												
		AlmostEmpty		=> AlmostEmpty			,
		Empty			=> Empty				,
		RdEn			=> RdEn					,
		RdData			=> RdData				,
												
		PageSel			=> PageSel				,
												
		Hsync_out   	=> wVGA_Hsync			,
		Vsync_out   	=> wVGA_Vsync  			,
		Blank_out   	=> wVGA_Blank  			,
		Red_out			=> wVGA_Red				,
		Green_out		=> wVGA_Green			,
		Blue_out		=> wVGA_Blue				
	);
	
	u_Fifo16i32o : Fifo16i32o
	Port map (
		rst			 	=> RstB				  	,
		wr_clk		 	=> Clk100			  	,
		wr_en		 	=> UserDataValid	  	,
		din			 	=> UserDataIn		  	,
		wr_data_count	=> WrCnt2			  	,
		rd_clk		 	=> Clk25			  	,
		rd_en		 	=> RdEn				  	,
		dout		 	=> RdData			  	,
		rd_data_count	=> open				  	,
		almost_empty	=> AlmostEmpty			,
		empty		 	=> Empty			  	,
		full		 	=> open	
	);
	
	u_Arbiter : Arbiter
	Port map (
		Clk				=> Clk100			  	,
		RstB			=> RstB				  	,
		
		CmdReq			=> UserCmdReq		  	,
		Cmd				=> UserCmd			  	,
		Busy			=> UserBusy			  	,
		Addr			=> UserAddr			  	,
		
		CmdReq1			=> CmdReq1			  	,
		Cmd1			=> Cmd1	  			  	,
		Busy1			=> Busy1  			  	,
		Addr1			=> Addr1  			  	,
		
		CmdReq2			=> CmdReq2			  	,
		Cmd2			=> Cmd2	  			  	,
		Busy2			=> Busy2  			  	,
		Addr2			=> Addr2	
	);
	
	u_VideoConfig : VideoConfig
	Port map (
		Clk				=> Clk100				,
		RstB			=> RstB					,
		ConfigRdy		=> ConfigRdy			,
		scl				=> scl					,
		sda				=> sda	
	);

End Architecture rtl;