----------------------------------------------------------------------------------
----------------------------------------------------------------------------------

-- Filename     UserIf.vhd

-- Title        User Interface to sdrm.vhd

--

-- Company      Design Gateway Co., Ltd.

-- Project      JAMPII

-- PJ No.       RD0505

-- Syntax       VHDL

-- Note         

--

--

-- Version      1.00

-- Author       P.Kosol

-- Date         2008/09/08

-- Remark       New Creation

----------------------------------------------------------------------------------
----------------------------------------------------------------------------------
Library IEEE;
USE		IEEE.STD_LOGIC_1164.all;
USE		IEEE.STD_LOGIC_UNSIGNED.all;
USE		IEEE.NUMERIC_STD.all;

Entity	UserIf is
Port(
		Clk 				: in 	std_logic;
		RstB 				: in 	std_logic;

		-- User Interface
		UserCmd				: in	std_logic;-- '0': Write, '1': Read
		UserCmdReq			: in	std_logic;
		UserAddr			: in	std_logic_vector(14 downto 0);
		UserBusy			: out	std_logic;
		UserDataReq			: out	std_logic;
		UserDataOut			: in	std_logic_vector(15 downto 0);
		UserDataValid		: out	std_logic;
		UserDataIn			: out	std_logic_vector(15 downto 0);
		
		-- Sdrm		
		WeRn 				: out 	std_logic;
		DataAddrN	 		: out 	std_logic;
		AddrIn				: out	std_logic_vector(31 downto 0);
		DataIn				: out	std_logic_vector(15 downto 0);
		DataOut				: in	std_logic_vector(15 downto 0)
	);
End Entity UserIf;

Architecture rtl of UserIf is

----------------------------------------------------------------------------------	
-- Constant Declaration
----------------------------------------------------------------------------------

	type state is (	
						stPowerOn,
						stPrecharge,
						stWtPrecharge,
						stAutoRefresh,
						stWtAutoRefresh,
						stLoadModeReg,
						stWtLoadModeReg,
						stRefresh,
						stWtRefresh,
						stWtCmd,
						stCmdWrite,
						stWriteData,
						stWtWriteData,
						stCmdRead,
						stReadData,
						stWtReadData,
						stPrechargeFP,
						stWtPrechargeFP
				);

	signal	rState		:	state;					
				  
----------------------------------------------------------------------------------	
-- Signal Declaration
----------------------------------------------------------------------------------

	-- Internal
	signal	rRefCnt			: std_logic_vector(14 downto 0);
	signal	rARCheck		: std_logic;
	signal	rWaitCnt		: std_logic_vector( 3 downto 0);
	signal	rShift14		: std_logic_vector(13 downto 0);

	-- User interface	
	signal	rUserBusy		: std_logic;
	signal	rUserDataReq	: std_logic;
	signal	rUserLength		: std_logic_vector( 8 downto 0);
	signal	rUserDataValid	: std_logic;
	signal	rUserDataIn		: std_logic_vector(15 downto 0);

	-- Sdrm interface	
	signal	rWeRn			: std_logic;
	signal	rDataAddrN		: std_logic;
	signal	rAddrIn			: std_logic_vector(31 downto 0);
	signal	rDataIn			: std_logic_vector(15 downto 0);
	signal	rWeRn1			: std_logic;
	signal	rDataAddrN1		: std_logic;
	signal	rAddrIn1		: std_logic_vector(31 downto 0);
	signal	rDataIn1		: std_logic_vector(15 downto 0);
	signal	rDataOut		: std_logic_vector(15 downto 0);
	
Begin

----------------------------------------------------------------------------------	
-- Output Assignment
----------------------------------------------------------------------------------

	WeRn			<=	rWeRn1;
	DataAddrN		<=	rDataAddrN1;
	AddrIn			<=	rAddrIn1;	
	DataIn			<=	rDataIn;
	UserBusy		<=	rUserBusy;
	UserDataReq		<=	rUserDataReq;
	UserDataValid	<=	rShift14(13) ;
	UserDataIn		<=	rDataOut;

----------------------------------------------------------------------------------	
-- DFF
----------------------------------------------------------------------------------	

	u_State : Process (RstB, Clk) Is
	Begin
		if ( RstB='0' ) then
			rState	<=	stPowerOn;	
		elsif ( rising_edge(Clk) ) then
			case (rState) is
			
				when stPowerOn			=>
					if ( rRefCnt(14)='1' ) then
						rState	<= stPrecharge;
					else
						rState	<= stPowerOn;
					end if;
				
				when stPrecharge		=>
					rState	<= stWtPrecharge;
				
				when stWtPrecharge		=>
					if ( rWaitCnt(1)='1' ) then
						rState	<= stAutoRefresh;
					else
						rState	<= stWtPrecharge;
					end if;
				
				when stAutoRefresh		=>
					rState	<= stWtAutoRefresh;
					
				when stWtAutoRefresh	=>
					if ( rWaitCnt(3)='1' ) then
						if ( rARCheck='0' ) then
							rState	<= stLoadModeReg;
						else
							rState	<= stAutoRefresh;
						end if;
					else
						rState	<= stWtAutoRefresh;
					end if;
				
				when stLoadModeReg		=>
					rState	<= stWtLoadModeReg;
					
				when stWtLoadModeReg	=>
					if ( rWaitCnt(1)='1' ) then
						rState	<= stWtCmd;
					else
						rState	<= stWtLoadModeReg;
					end if;

				when stWtCmd			=>					
					if ( rRefCnt(8)='1' ) then
						rState	<= stRefresh;
					elsif ( UserCmdReq='1' ) then
						if ( UserCmd='0' ) then
							rState	<= stCmdWrite;
						else
							rState	<= stCmdRead;
						end if;
					else	
						rState	<= stWtCmd;
					end if;

				when stRefresh			=>
					rState		<= stWtRefresh;

				when stWtRefresh		=>
					if ( rRefCnt(3)='1' ) then
						rState	<= stWtCmd;
					else
						rState	<= stWtRefresh;					
					end if;
										
				when stCmdWrite			=>
					rState		<= stWriteData;
					
				when stWriteData		=>
					if ( rUserLength="000000000" ) then
						rState	<= stWtWriteData;
					else
						rState	<= stWriteData;
					end if;
				
				when stWtWriteData		=>
					if ( rWaitCnt(2 downto 0)="111" ) then
						rState	<= stPrechargeFP;
					else
						rState	<= stWtWriteData;
					end if;
				
				when stCmdRead			=>
					rState		<= stReadData;
					
				when stReadData			=>
					if ( rUserLength="000000000" ) then
						rState	<= stWtReadData;
					else
						rState	<= stReadData;
					end if;
					
				when stWtReadData		=>
					if ( rWaitCnt(2 downto 0)="111" ) then
						rState	<= stPrechargeFP;
					else
						rState	<= stWtReadData;
					end if;
				
				when stPrechargeFP		=>
					rState		<= stWtPrechargeFP;
					
				when stWtPrechargeFP	=>
					if ( rWaitCnt(1)='1' ) then
						rState	<= stRefresh;
					else
						rState	<= stWtPrechargeFP;
					end if;
				
				when others				=>
					rState		<= stPowerOn;
				
			end case;			
		end if;
	End Process u_State;

	u_rRefCnt : Process (RstB, Clk) Is
	Begin
		if ( RstB='0' ) then
			rRefCnt	<= (others => '0');	
		elsif ( rising_edge(Clk) ) then
			if ( rState=stAutoRefresh or rState=stRefresh ) then
				rRefCnt	<= (others => '0');
			else
				rRefCnt	<=	rRefCnt + 1;
			end if;
		end if;
	End Process u_rRefCnt;

	u_rWaitCnt : Process (RstB, Clk) Is
	Begin
		if ( RstB='0' ) then
			rWaitCnt	<=	"0000";
		elsif ( rising_edge(Clk) ) then
			if ( rState=stWtPrecharge or rState=stWtLoadModeReg or rState=stWtAutoRefresh
			or   rState=stWtWriteData or rState=stWtReadData    or rState=stWtPrechargeFP ) then
				rWaitCnt	<= rWaitCnt + 1;
			else
				rWaitCnt	<= "0000";
			end if;
		end if;
	End Process u_rWaitCnt;

	u_rARCheck : Process (RstB, Clk) Is
	Begin
		if ( RstB='0' ) then
			rARCheck	<= '0';
		elsif ( rising_edge(Clk) ) then
			if ( rState=stAutoRefresh ) then
				rARCheck	<= not rARCheck;
			end if; 
		end if;
	End Process u_rARCheck;

---------------------------------------------------------------------------
-- User interface
	
	u_rUserBusy : Process (RstB, Clk) Is
	Begin
		if ( RstB='0' ) then
			rUserBusy	<=	'1';
		elsif ( rising_edge(Clk) ) then
			if ( rState=stWtCmd ) then
				if ( rRefCnt(8)='0' and UserCmdReq='1' ) then
					rUserBusy	<= '1';
				else
					rUserBusy	<= '0';
				end if;
			else
				rUserBusy	<= rUserBusy;
			end if;
		end if;
	End Process u_rUserBusy;

	u_rUserLength : Process (RstB, Clk) Is
	Begin
		if ( RstB='0' ) then
			rUserLength	<= "000000000";
		elsif ( rising_edge(Clk) ) then
			if ( rState=stCmdWrite or rState=stCmdRead ) then
				rUserLength	<= "111111111";
			elsif ( rState=stWriteData or rState=stReadData ) then
				rUserLength	<= rUserLength - 1;
			else
				rUserLength	<= rUserLength;
			end if;
		end if;
	End Process u_rUserLength;

	u_rUserDataReq : Process (RstB, Clk) Is
	Begin
		if ( RstB='0' ) then
			rUserDataReq	<= '0';
		elsif ( rising_edge(Clk) ) then
			if ( rState=stCmdWrite ) then
				rUserDataReq	<= '1';
			elsif ( rState=stWriteData ) then
				if ( rUserLength = "000000000" ) then	
					rUserDataReq	<= '0';
				else
					rUserDataReq	<= '1';
				end if;
			else
				rUserDataReq	<= '0';
			end if;
		end if;
	End Process u_rUserDataReq;
	
	u_rUserDataValid : Process (RstB, Clk) Is
	Begin
		if ( RstB='0' ) then
			rUserDataValid	<=	'0';
		elsif ( rising_edge(Clk) ) then
			if ( rState=stReadData ) then
				rUserDataValid	<= '1';
			else
				rUserDataValid	<= '0';
			end if;
		end if;
	End Process u_rUserDataValid;
	
	u_rShift14 : Process (RstB, Clk) Is
	Begin
		if ( RstB='0' ) then
			rShift14	<=	(others =>'0');
		elsif ( rising_edge(Clk) ) then
			rShift14	<=	rShift14(12 downto 0) & rUserDataValid;
		end if;
	End Process u_rShift14;

---------------------------------------------------------------------------
-- Sdrm interface

	u_rWeRn1 : Process (RstB, Clk) Is
	Begin
		if ( RstB='0' ) then
			rWeRn1		<= '0';
			rDataAddrN1	<= '1';
			rAddrIn1	<= (others=>'0');
			rDataIn1	<= (others=>'0');
		elsif ( rising_edge(Clk) ) then
			rWeRn1		<= rWeRn;
			rDataAddrN1	<= rDataAddrN;
			rAddrIn1	<= rAddrIn;
			rDataIn1	<= rDataIn;
		end if;
	End Process u_rWeRn1;

	u_rWeRn : Process (RstB, Clk) Is
	Begin
		if ( RstB='0' ) then
			rWeRn	<= '0';
		elsif ( rising_edge(Clk) ) then
			if ( rState=stPrecharge or rState=stAutoRefresh or rState=stLoadModeReg
			or   rState=stRefresh   or rState=stCmdWrite    or rState=stWriteData
			or   rState=stPrechargeFP ) then
				rWeRn	<= '1';
			else
				rWeRn	<= '0';
			end if;
		end if;
	End Process u_rWeRn;
	
	u_rDataAddrN : Process (RstB, Clk) Is
	Begin
		if ( RstB='0' ) then
			rDataAddrN	<=	'1';
		elsif ( rising_edge(Clk) ) then
			if ( rState=stPrecharge or rState=stAutoRefresh or rState=stLoadModeReg
			or   rState=stRefresh   or rState=stCmdWrite    or rState=stCmdRead
			or   rState=stPrechargeFP ) then
				rDataAddrN	<=	'0';
			else
				rDataAddrN	<=	'1';
			end if;
		end if;
	End Process u_rDataAddrN;
	
	u_rAddrIn : Process (RstB, Clk) Is
	Begin
		if ( RstB='0' ) then
			rAddrIn	<= (others => '0');
		elsif ( rising_edge(Clk) ) then
			case (rState) is
				when stPrecharge	=> rAddrIn	<= x"20340400";
				when stAutoRefresh	=> rAddrIn	<= x"38000000";
				when stLoadModeReg	=> rAddrIn	<= x"1000dc00";
				when stRefresh		=> rAddrIn	<= x"38000000";
				when stCmdWrite		=> rAddrIn	<= "0000000" & UserAddr & "0000000000";
				when stCmdRead		=> rAddrIn	<= "0000000" & UserAddr & "0000000000";
				when stPrechargeFP	=> rAddrIn	<= x"20340400";
				when others			=> rAddrIn	<= (others => '0');
			end case;
		end if;
	End Process u_rAddrIn;
	
	u_rDataIn : Process (RstB, Clk) Is
	Begin
		if ( RstB='0' ) then
			rDataIn		<= (others => '0');
		elsif ( rising_edge(Clk) ) then
			rDataIn		<= UserDataOut;
		end if;
	End Process u_rDataIn;
	
	u_rDataOut : Process (RstB, Clk) Is
	Begin
		if ( RstB='0' ) then
			rDataOut 	<= (others => '0');
		elsif ( rising_edge(Clk) ) then
			rDataOut	<= DataOut;
		end if;
	End Process u_rDataOut;
	
End Architecture rtl;