----------------------------------------------------------------------------------

----------------------------------------------------------------------------------


-- Filename     VideoConfig.vhd


-- Title        VideoConfig


--


-- Company      Design Gateway Co., Ltd.


-- Project      JAMPII


-- PJ No.       RD0505


-- Syntax       VHDL


-- Note         


--


--


-- Version      1.00


-- Author       P.Kosol


-- Date         2008/10/06


-- Remark       New Creation


----------------------------------------------------------------------------------

----------------------------------------------------------------------------------

Library	IEEE;
Use		IEEE.STD_LOGIC_1164.all;
Use		IEEE.STD_LOGIC_ARITH.all;

Entity VideoConfig is
Port(
		Clk    		: in 	std_logic;
		RstB 		: in 	std_logic;
		
		ConfigRdy	: out	std_logic;
		
		scl			: inout	std_logic;
		sda			: inout	std_logic
	);
End Entity VideoConfig;

Architecture rtl of VideoConfig is
----------------------------------------------------------------------------------
-- Component Declaration
----------------------------------------------------------------------------------
	Component ConfigStorage is
	Port (
		Clk    		: in 	std_logic;
		RstB 		: in 	std_logic;
		
		UserBusy	: in	std_logic;	
		UserStart	: out	std_logic;
		UserDevAddr	: out	std_logic_vector( 6 downto 0 );
		UserRegAddr	: out	std_logic_vector( 7 downto 0 );
		UserData	: out	std_logic_vector(15 downto 0 );
		
		ConfigRdy	: out	std_logic			
	);
	End Component ConfigStorage;

	Component Master
	Port (
		Clk    		: in 	std_logic;
		RstB 		: in 	std_logic;
				
		UserStart	: in	std_logic;
		UserDevAddr	: in	std_logic_vector( 6 downto 0 );
		UserRegAddr	: in	std_logic_vector( 7 downto 0 );
		UserData	: in	std_logic_vector(15 downto 0 );
		UserBusy	: out	std_logic;
		
		ena    		: out 	std_logic;
		clk_cnt 	: out 	unsigned(15 downto 0);
		start		: out 	std_logic;
		read		: out 	std_logic;
		write		: out 	std_logic;
		stop		: out 	std_logic;
		ack_in 		: out	std_logic;
		din    		: out 	std_logic_vector( 7 downto 0);
		cmd_ack  	: in	std_logic	
	);
	End Component;
	
	Component i2c_master_byte_ctrl
	Port(
		clk    		: in 	std_logic;
		rst    		: in 	std_logic;
		nReset 		: in 	std_logic;

		ena    		: in 	std_logic;
		clk_cnt 	: in 	unsigned(15 downto 0);
		start		: in 	std_logic;
		stop		: in 	std_logic;
		read		: in 	std_logic;
		write		: in 	std_logic;
		ack_in 		: in 	std_logic;
		din    		: in 	std_logic_vector(7 downto 0);

		cmd_ack 	: out 	std_logic; 
		ack_out 	: out 	std_logic;
		i2c_busy	: out 	std_logic; 
		i2c_al  	: out 	std_logic; 
		dout    	: out 	std_logic_vector(7 downto 0);

		scl_i   	: in 	std_logic; 
		scl_o   	: out 	std_logic;
		scl_oen 	: out 	std_logic;
		sda_i   	: in 	std_logic; 
		sda_o   	: out 	std_logic;
		sda_oen 	: out 	std_logic
	);
	End Component;

----------------------------------------------------------------------------------
-- Signal Declaration
----------------------------------------------------------------------------------
	signal	rst			: std_logic;

	signal	UserStart	: std_logic;
	signal	UserDevAddr	: std_logic_vector( 6 downto 0 );
	signal	UserRegAddr	: std_logic_vector( 7 downto 0 );
	signal	UserData	: std_logic_vector(15 downto 0 );
	signal	UserBusy	: std_logic;
	
	signal	ena    		: std_logic;
	signal	clk_cnt 	: unsigned(15 downto 0);
	signal	start		: std_logic;
	signal	read		: std_logic;
	signal	write		: std_logic;
	signal	stop		: std_logic;
	signal	ack_in 		: std_logic;
	signal	din    		: std_logic_vector( 7 downto 0);
	signal	cmd_ack  	: std_logic;
	
	signal	scl_i   	: std_logic;
	signal	scl_o   	: std_logic;
	signal	scl_oen 	: std_logic;
	signal	sda_i   	: std_logic;
	signal	sda_o   	: std_logic;
	signal	sda_oen 	: std_logic;

Begin
----------------------------------------------------------------------------------
-- Output Assignment	
----------------------------------------------------------------------------------
	scl		<=	scl_o when ( scl_oen='0' ) else 'Z';
	sda		<=	sda_o when ( sda_oen='0' ) else 'Z';
	scl_i	<=	scl;
	sda_i	<=	sda;
	
	rst		<=	not RstB;

----------------------------------------------------------------------------------
-- Component Mapping	
----------------------------------------------------------------------------------		
	u_ConfigStorage : ConfigStorage
	Port Map(
		Clk    		=>	Clk    		,
		RstB 		=>	RstB 		,

		UserBusy	=>	UserBusy	,
		UserStart	=>	UserStart	,
		UserDevAddr	=>	UserDevAddr	,
		UserRegAddr	=>	UserRegAddr	,
		UserData	=>	UserData	,
		ConfigRdy	=>	ConfigRdy	
	);					
	
	u_Master : Master
	Port Map(
		Clk			=>	Clk			,			    		
		RstB 		=>	RstB 		,
									
		UserStart	=>	UserStart	,
		UserDevAddr	=>	UserDevAddr	,
		UserRegAddr	=>	UserRegAddr	,
		UserData	=>	UserData	,
		UserBusy	=>	UserBusy	,
		ena    		=>	ena    		,
		clk_cnt 	=>	clk_cnt 	,
									
		start		=>	start		,
		read		=>	read		,
		write		=>	write		,
		stop		=>	stop		,
		ack_in 		=>	ack_in 		,
		din    		=>	din    		,
		cmd_ack  	=>	cmd_ack  		
	);								
									
	u_byte_ctrl : i2c_master_byte_ctrl
	Port Map(
		clk    		=>	Clk			,    		
		rst    		=>	rst    		,
		nReset 		=>	RstB 		,

		ena    		=>	ena    		,
		clk_cnt 	=>	clk_cnt 	,
		start		=>	start		,
		stop		=>	stop		, 
		read		=>	read		,
		write		=>	write		,
		ack_in 		=>	ack_in 		,
		din    		=>	din    		,

		cmd_ack 	=>	cmd_ack	 	,
		ack_out 	=>	open	 	,
		i2c_busy	=>	open		,
		i2c_al  	=>	open	  	,
		dout    	=>	open    	,

		scl_i   	=>	scl_i   	,
		scl_o   	=>	scl_o   	,
		scl_oen 	=>	scl_oen 	,
		sda_i   	=>	sda_i   	,
		sda_o   	=>	sda_o   	,
		sda_oen    	=>	sda_oen  
	);     	

End Architecture rtl;
