library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.numeric_STD.all;
library UNISIM;
use UNISIM.vcomponents.all;

entity sdrm is
port (
	Reset 				: in 	std_logic;
	Clkp 				: in 	std_logic;
	-- User interface
	we_rn 				: in 	std_logic;
	data_addr_n 		: in 	std_logic;
	AddrIn				: in	std_logic_vector(31 downto 0);
	DataIn				: in	std_logic_vector(15 downto 0);
	DataOut				: out	std_logic_vector(15 downto 0);
	-- Sdram interface
	sd_data				: inout std_logic_vector(15 downto 0);
	sd_add				: out	std_logic_vector(12 downto 0);
	sd_ras 				: out 	std_logic;
	sd_cas 				: out 	std_logic;
	sd_we 				: out 	std_logic;
	sd_ba				: out	std_logic_vector(1 downto 0);
	Clk_Sdram 			: out 	std_logic;
	sd_cke 				: out 	std_logic;
	sd_cs				: out	std_logic;
	sd_dqm				: out	std_logic_vector(1 downto 0)
);
end entity sdrm;

architecture sdrm_arch of sdrm is 
--------------------------------------------------------------------------------
-- Constant Declaration
--------------------------------------------------------------------------------
	constant DATA_MSB 		: integer := 15;
	constant ADDR_MSB 		: integer := 31;

	constant RcdMax   		: unsigned( 1 downto 0) := "01";
	constant CasMax   		: unsigned( 1 downto 0) := "10";
	constant BurstMax   	: std_logic_vector( 8 downto 0) := "111111111";--Full Page 512 words
	constant KiMax   		: unsigned( 3 downto 0) := "0101";
	constant RefMax			: unsigned(15 downto 0) := x"04a0";

--------------------------------------------------------------------------------
-- Signal Declaration
--------------------------------------------------------------------------------
	signal	RstB			: std_logic;
	signal	sd_add_op		: std_logic_vector(12 downto 0);
	signal	sd_data_i		: std_logic_vector(DATA_MSB downto 0);
	signal	sd_data_reg		: std_logic_vector(DATA_MSB downto 0);
	signal	sd_ba_op		: std_logic_vector( 1 downto 0);
	signal	ready_o 		: std_logic;
	signal	sd_data_o		: std_logic_vector(DATA_MSB downto 0);
	signal	sd_data_t		: std_logic_vector(DATA_MSB downto 0);
	signal	sd_data_R		: std_logic_vector(DATA_MSB downto 0);
	signal	sd_add_o		: std_logic_vector(12 downto 0);
	signal	sd_ba_o			: std_logic_vector( 1 downto 0);
	signal	sd_cas_o 		: std_logic;
	signal	sd_ras_o 		: std_logic;
	signal	sd_we_o 		: std_logic;
	signal	sd_doe_n 		: std_logic_vector( 3 downto 0);
	signal	sd_doe_np 		: std_logic_vector( 3 downto 0);
	signal	Clk 			: std_logic;
	signal	sd_ras_op 		: std_logic;
	signal	sd_cas_op 		: std_logic;
	signal	sd_we_op 		: std_logic;
	signal	write_st 		: std_logic;
	signal	AD_tri 			: std_logic;
	signal	Add_reg 		: std_logic_vector(ADDR_MSB downto 0);
	signal	AD_reg			: std_logic_vector(DATA_MSB downto 0);
	signal	Act_st 			: std_logic_vector(2 downto 0);
	signal	data_addr_n_reg : std_logic;
	signal	we_rn_reg 		: std_logic;
	signal	we_rn_i 		: std_logic;
	signal	kid 			: std_logic;
	signal	auto_ref_out 	: std_logic;
	signal	auto_ref_in 	: std_logic;
	signal	rcd_end 		: std_logic;
	signal	AddrIn_reg		: std_logic_vector(ADDR_MSB downto 0);
	signal	DataOut_reg		: std_logic_vector(DATA_MSB downto 0);

--------------------------------------------------------------------------------
-- Component Declaration
--------------------------------------------------------------------------------
component sys_int 
generic (
	ADDR_MSB 		: integer := 31;
	DATA_MSB 		: integer := 15;
	CYCLE 			: integer := 8;
	HALF_CYCLE 		: integer := 4
);
port (
	RstB			: in	std_logic;
	Add_reg 		: out 	std_logic_vector(ADDR_MSB downto 0);
	sd_data_reg		: out	std_logic_vector(DATA_MSB downto 0);
	Act_st 			: out 	std_logic_vector(2 downto 0);
	write_st 		: out 	std_logic;
	AD_reg			: in	std_logic_vector(DATA_MSB downto 0);
	Clk_i 			: in 	std_logic;
	data_addr_n_i 	: in 	std_logic;
	we_rn_i 		: in 	std_logic;
	data_addr_n_reg : in 	std_logic;
	we_rn_reg 		: in 	std_logic;
	AddrIn_reg		: in	std_logic_vector(ADDR_MSB downto 0)
);
end component;

component sdrm_t 
port (
	RstB			: in	std_logic;
	sd_add_o		: out 	std_logic_vector(12 downto 0);
	sd_ras_o		: out 	std_logic;
	sd_cas_o		: out 	std_logic;
	sd_we_o			: out 	std_logic;
	sd_ba_o			: out	std_logic_vector( 1 downto 0);
	ready_o			: out 	std_logic;
	kid				: out 	std_logic;
	auto_ref_out	: out 	std_logic;
	rcd_end			: out 	std_logic;
	sd_doe_n		: out 	std_logic_vector( 3 downto 0);
	AD_tri			: out 	std_logic;
	write_st		: in 	std_logic;
	auto_ref_in		: in 	std_logic;
	Clk_i			: in 	std_logic;
	Clk_j			: in 	std_logic;
	Act_st			: in 	std_logic_vector( 2 downto 0);
	Add_reg			: in	std_logic_vector(24 downto 1);
	rcd_c_max		: in 	unsigned( 1 downto 0);
	cas_lat_max		: in 	unsigned( 1 downto 0);
	burst_max		: in 	std_logic_vector( 8 downto 0);
	ki_max			: in 	unsigned( 3 downto 0);
	ref_max			: in 	unsigned(15 downto 0)
);
end component;

begin
---------------------------------------------------------------------------------
-- Output Assignment	
---------------------------------------------------------------------------------	
	RstB					<= not Reset;
	DataOut					<= DataOut_reg;
	auto_ref_in 			<= auto_ref_out;
	sd_cke					<= '1';
	sd_cs					<= '0';
	sd_dqm					<= "00";
	Clk_Sdram				<= not Clkp;
	sd_add(12 downto 0)		<= sd_add_o;
	sd_ba(1 downto 0)		<= sd_ba_o;
	sd_ras					<= sd_ras_o;
	sd_cas					<= sd_cas_o;
	sd_we					<= sd_we_o;
	we_rn_i					<= we_rn;	
	sd_data_i(15 downto 0)	<= sd_data(15 downto 0) after 3 ns;
	iod : For i in 0 to 15 generate
		sd_data(i)		<= sd_data_o(i) when sd_data_t(i)='0' else 'Z';
	end generate;

---------------------------------------------------------------------------------
-- Component Mapping	
---------------------------------------------------------------------------------	
--Instantiation of sub-level modules
sdrm_t_int : sdrm_t 
port map (	
	RstB			=> RstB					,
	Clk_i			=> Clkp					,
	sd_ras_o 		=> sd_ras_op			, 
	sd_cas_o 		=> sd_cas_op			, 
	sd_we_o 		=> sd_we_op				,
	sd_add_o 		=> sd_add_op			, 
	sd_ba_o 		=> sd_ba_op				, 
	sd_doe_n 		=> sd_doe_np			, 
	ready_o 		=> ready_o				, 
	Add_reg			=> Add_reg(24 downto 1) ,
	kid 			=> kid					,
	auto_ref_in 	=> auto_ref_in			, 
	auto_ref_out 	=> auto_ref_out			,
	Act_st 			=> Act_st				, 
	Clk_j			=> Clkp					,
	write_st 		=> write_st				,
	rcd_end 		=> rcd_end				, 
	AD_tri 			=> AD_tri				,
	rcd_c_max 		=> RcdMax				,  
	cas_lat_max 	=> CasMax   			,
	burst_max 		=> BurstMax				,
	ki_max 			=> KiMax   				,
	ref_max			=> RefMax  
);
   
sys_int_int : sys_int 
port map (
	RstB			=> RstB					,
	Clk_i			=> Clkp					,
	data_addr_n_reg => data_addr_n_reg		,
	data_addr_n_i 	=> data_addr_n			, 
	we_rn_i 		=> we_rn_i				,
	we_rn_reg 		=> we_rn_reg			, 
	AD_reg 			=> AD_reg				,
	Act_st 			=> Act_st				, 
	write_st 		=> write_st				,
	Add_reg 		=> Add_reg				, 
	sd_data_reg 	=> sd_data_reg			, 
	AddrIn_reg		=> AddrIn_reg
);

---------------------------------------------------------------------------------
-- DFF	
---------------------------------------------------------------------------------

process(Clkp, RstB)
begin
	if (RstB = '0') then
		sd_data_o		<= x"ffff";
		sd_data_t 		<= x"ffff";
		sd_data_R 		<= x"0000";
		DataOut_reg 	<= x"0000";
		AD_reg 			<= x"0000";
		data_addr_n_reg <= '1';
		we_rn_reg 		<= '0';
		AddrIn_reg 		<= x"00000000";
	elsif rising_edge(Clkp) then
		sd_data_o <= transport sd_data_reg after 3 ns;
		sd_data_t(15 downto 8) <= transport (sd_doe_n(1) & sd_doe_n(1) & sd_doe_n(1) & 
						sd_doe_n(1) & sd_doe_n(1) & sd_doe_n(1) & 
					sd_doe_n(1) & sd_doe_n(1) ) after 3 ns;
		sd_data_t(7 downto 0) <= transport (sd_doe_n(0) & sd_doe_n(0) & sd_doe_n(0) & 
						sd_doe_n(0) & sd_doe_n(0) & sd_doe_n(0) & 
					sd_doe_n(0) & sd_doe_n(0) ) after 3 ns;

		sd_data_R 		<= sd_data_i;
		DataOut_reg 	<= sd_data_R;
		AD_reg 			<= DataIn;
		data_addr_n_reg <= data_addr_n;
		we_rn_reg 		<= we_rn_i;
		AddrIn_reg 		<= AddrIn;
	end if;
end process;

process (Clkp, RstB)
begin
	if (RstB = '0') then
		sd_add_o(12 downto 0) 	<= "0000000000000";
		sd_ba_o					<= "00";
		sd_ras_o 				<= '1';
		sd_cas_o 				<= '1';
		sd_we_o 				<= '1';
		sd_doe_n 				<= X"f";
	elsif rising_edge(Clkp) then
		sd_add_o(12 downto 0) 	<= transport sd_add_op(12 downto 0) after 3 ns;
		sd_ba_o					<= transport sd_ba_op after 3 ns;
		sd_ras_o 				<= transport sd_ras_op after 3 ns;
		sd_cas_o 				<= transport sd_cas_op after 3 ns;
		sd_we_o 				<= transport sd_we_op after 3 ns;
		sd_doe_n 				<= transport sd_doe_np after 3 ns;
	end if;
end process;

end architecture sdrm_arch;



