library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.numeric_STD.all;
library UNISIM;
use UNISIM.vcomponents.all;

entity sys_int is
generic (
	ADDR_MSB 		: integer := 31;
	DATA_MSB 		: integer := 15;
	CYCLE 			: integer := 8;
	HALF_CYCLE 		: integer := 4
);
port (
	RstB			: in	std_logic;
	Add_reg 		: out 	std_logic_vector(ADDR_MSB downto 0);
	sd_data_reg		: out	std_logic_vector(DATA_MSB downto 0);
	Act_st 			: out 	std_logic_vector(2 downto 0);
	write_st 		: out 	std_logic;
	AD_reg			: in	std_logic_vector(DATA_MSB downto 0);
	Clk_i 			: in 	std_logic;
	data_addr_n_i 	: in 	std_logic;
	we_rn_i 		: in 	std_logic;
	data_addr_n_reg : in 	std_logic;
	we_rn_reg 		: in 	std_logic;
	AddrIn_reg		: in	std_logic_vector(ADDR_MSB downto 0)
);
end entity sys_int;

architecture sys_int_arch of sys_int is

-------------------------------------------------------------------------------
-- Component Declaration
-------------------------------------------------------------------------------
component SRL16
 port (
   Q 	: out 	std_logic;
   A0 	: in 	std_logic;
   A1 	: in 	std_logic;
   A2 	: in 	std_logic;
   A3 	: in 	std_logic;
   D 	: in 	std_logic;
   CLK 	: in 	std_logic
);
end component;

-------------------------------------------------------------------------------
-- Signal Declaration
-------------------------------------------------------------------------------
signal addr 			: std_logic_vector(ADDR_MSB downto 0);
signal Data_reg			: std_logic_vector(DATA_MSB downto 0);
signal ld_cnt_reg 		: std_logic;
signal sd_data_reg_int	: std_logic_vector(DATA_MSB downto 0 );
signal Act_st_int 		: std_logic_vector(2 downto 0);

begin
-------------------------------------------------------------------------------
-- Output Assignment
-------------------------------------------------------------------------------	
write_st 	<= we_rn_i;
sd_data_reg <= sd_data_reg_int;
Act_st 		<= Act_st_int;
	
-------------------------------------------------------------------------------
-- DFF
-------------------------------------------------------------------------------
process (Clk_i, data_addr_n_reg)
begin
	if rising_edge(Clk_i) then
		if (data_addr_n_reg = '0') then
			case AddrIn_reg(29 downto 28) is
				when "00" =>
					Act_st_int <= "100";
				when "01" =>
					Act_st_int <= "101";
				when "10" =>
					Act_st_int <= "110";
				when "11" =>
					Act_st_int <= "111";
				when others =>
					Act_st_int <= "000";
			end case;
		else 
			Act_st_int <= "000";
		end if;
	end if;
end process;


process (Clk_i, RstB)
begin
	if (RstB = '0') then
		ld_cnt_reg <= '0';
		Add_reg 	<= X"00000000";
		Data_reg	<= x"0000";
	elsif rising_edge(Clk_i) then
		ld_cnt_reg <= (Act_st_int(2) and Act_st_int(1) and not(Act_st_int(0)) );
		if (data_addr_n_reg = '1') then
			Data_reg 	<= AD_reg;
		else
			Add_reg 	<= AddrIn_reg;
		end if;
	end if;
end process;

--delay Data_reg  by 4 clock periods
SRL16_0 : SRL16 port map (Q => sd_data_reg_int(0), A0 => '1', A1 => '1', A2 => '0', A3 => '0', 
		D => Data_reg(0), CLK => Clk_i);
SRL16_1 : SRL16 port map (Q => sd_data_reg_int(1), A0 => '1', A1 => '1', A2 => '0', A3 => '0', 
		D => Data_reg(1), CLK => Clk_i);
SRL16_2 : SRL16 port map (Q => sd_data_reg_int(2), A0 => '1', A1 => '1', A2 => '0', A3 => '0', 
		D => Data_reg(2), CLK => Clk_i);
SRL16_3 : SRL16 port map (Q => sd_data_reg_int(3), A0 => '1', A1 => '1', A2 => '0', A3 => '0', 
		D => Data_reg(3), CLK => Clk_i);
SRL16_4 : SRL16 port map (Q => sd_data_reg_int(4), A0 => '1', A1 => '1', A2 => '0', A3 => '0', 
		D => Data_reg(4), CLK => Clk_i);
SRL16_5 : SRL16 port map (Q => sd_data_reg_int(5), A0 => '1', A1 => '1', A2 => '0', A3 => '0', 
		D => Data_reg(5), CLK => Clk_i);
SRL16_6 : SRL16 port map (Q => sd_data_reg_int(6), A0 => '1', A1 => '1', A2 => '0', A3 => '0', 
		D => Data_reg(6), CLK => Clk_i);
SRL16_7 : SRL16 port map (Q => sd_data_reg_int(7), A0 => '1', A1 => '1', A2 => '0', A3 => '0', 
		D => Data_reg(7), CLK => Clk_i);
SRL16_8 : SRL16 port map (Q => sd_data_reg_int(8), A0 => '1', A1 => '1', A2 => '0', A3 => '0', 
		D => Data_reg(8), CLK => Clk_i);
SRL16_9 : SRL16 port map (Q => sd_data_reg_int(9), A0 => '1', A1 => '1', A2 => '0', A3 => '0', 
		D => Data_reg(9), CLK => Clk_i);
SRL16_10 : SRL16 port map (Q => sd_data_reg_int(10), A0 => '1', A1 => '1', A2 => '0', A3 => '0', 
		D => Data_reg(10), CLK => Clk_i);
SRL16_11 : SRL16 port map (Q => sd_data_reg_int(11), A0 => '1', A1 => '1', A2 => '0', A3 => '0', 
		D => Data_reg(11), CLK => Clk_i);
SRL16_12 : SRL16 port map (Q => sd_data_reg_int(12), A0 => '1', A1 => '1', A2 => '0', A3 => '0', 
		D => Data_reg(12), CLK => Clk_i);
SRL16_13 : SRL16 port map (Q => sd_data_reg_int(13), A0 => '1', A1 => '1', A2 => '0', A3 => '0', 
		D => Data_reg(13), CLK => Clk_i);
SRL16_14 : SRL16 port map (Q => sd_data_reg_int(14), A0 => '1', A1 => '1', A2 => '0', A3 => '0', 
		D => Data_reg(14), CLK => Clk_i);
SRL16_15 : SRL16 port map (Q => sd_data_reg_int(15), A0 => '1', A1 => '1', A2 => '0', A3 => '0', 
		D => Data_reg(15), CLK => Clk_i);

end architecture sys_int_arch;











