----------------------------------------------------------------------------------
----------------------------------------------------------------------------------
-- Filename     TbSystem2.vhd
-- Title        SDRAM Controller
--
-- Company      Design Gateway Co., Ltd.
-- Project      JAMPII
-- PJ No.       RD0505
-- Syntax       VHDL
-- Note         
--
--
-- Version      1.00
-- Author       P.Kosol
-- Remark       New Creation
----------------------------------------------------------------------------------
----------------------------------------------------------------------------------
Library IEEE;
Use		IEEE.STD_LOGIC_1164.all;
Use		IEEE.STD_LOGIC_UNSIGNED.all;

Entity TbSystem2 is

End Entity TbSystem2;

Architecture test of TbSystem2 is

-------------------------------------------------------------------
-- Constant Declaration
-------------------------------------------------------------------

	constant 	period	:	TIME :=	10 ns;
	
-------------------------------------------------------------------
-- Component Declaration
-------------------------------------------------------------------

	Component System2 is
		Port(
			SysClk			: in	std_logic;
			RstB			: in	std_logic;

			sd_data			: inout std_logic_vector(15 downto 0);
			sd_add			: out	std_logic_vector(12 downto 0);
			sd_ras 			: out 	std_logic;
			sd_cas 			: out 	std_logic;
			sd_we 			: out 	std_logic;
			sd_ba			: out	std_logic_vector( 1 downto 0);
			Clk_Sdram 		: out 	std_logic;
			sd_cke 			: out 	std_logic;
			sd_cs			: out	std_logic;
			sd_dqm			: out	std_logic_vector( 1 downto 0);
			
			scl				: inout	std_logic;
			sda				: inout	std_logic;		

			VGA_Clk			: in	std_logic;
			VGA_nRst		: out	std_logic;
			VGA_PixClk		: out	std_logic;
			VGA_nPixClk		: out	std_logic;
			VGA_Hsync		: out	std_logic;
			VGA_Hsync_out	: out	std_logic;   
			VGA_Vsync		: out 	std_logic;
			VGA_Vsync_out	: out	std_logic;
			VGA_Blank		: out 	std_logic;   
			VGA_Red			: out	std_logic_vector( 7 downto 0 );		
			VGA_Green		: out	std_logic_vector( 7 downto 0 );	
			VGA_Blue 		: out	std_logic_vector( 7 downto 0 )
		);
	End Component System2;

	Component mt48lc16m16a2 is
		port(
			Clk				: in	std_logic;

			Addr			: in	std_logic_vector(12 downto 0);
			Ba				: in	std_logic_vector( 1 downto 0);
			Cke				: in	std_logic;
			Cs_n			: in	std_logic;
			Ras_n			: in	std_logic;
			Cas_n			: in	std_logic;
			We_n			: in	std_logic;
			Dqm				: in	std_logic_vector( 1 downto 0);
			Dq				: inout	std_logic_vector(15 downto 0)					
		);
	End Component mt48lc16m16a2;

	Component i2c_slave_model is
		Port (
			scl				: in	std_logic;
			sda				: inout	std_logic		
		);
	End Component i2c_slave_model;
			
-------------------------------------------------------------------
-- Signal Declaration
-------------------------------------------------------------------

	signal	RstB       		: std_logic;
	signal	SysClk        	: std_logic;

	signal  SdrDq      		: std_logic_vector(15 downto 0);
	signal	SdrClk	   		: std_logic;
	signal	SdrCke     		: std_logic;                      
	signal	SdrCs      		: std_logic;                      
	signal	SdrRas     		: std_logic;                      
	signal	SdrCas     		: std_logic;                      
	signal  SdrWe      		: std_logic;                      
	signal  SdrDqm     		: std_logic_vector( 1 downto 0);
	signal  SdrBa      		: std_logic_vector( 1 downto 0); 		
	signal	SdrAddr    		: std_logic_vector(12 downto 0);                       		
		
	signal	Failed			: std_logic;
	
	signal	scl				: std_logic;
	signal	sda				: std_logic;

	signal	VGA_Clk			: std_logic;
	signal	VGA_nRst		: std_logic;
	signal	VGA_PixClk		: std_logic;
	signal	VGA_nPixClk		: std_logic;
	signal	VGA_Hsync		: std_logic;                    
	signal	VGA_Hsync_out	: std_logic;
	signal	VGA_Vsync		: std_logic;                    
	signal	VGA_Vsync_out	: std_logic;
	signal  VGA_Blank		: std_logic;                    
	signal  VGA_Red			: std_logic_vector( 7 downto 0);
	signal  VGA_Green		: std_logic_vector( 7 downto 0);
	signal	VGA_Blue 		: std_logic_vector( 7 downto 0);
				
Begin
-------------------------------------------------------------------
-- Output Assignment
-------------------------------------------------------------------
	scl		<=  'H';
	sda		<=  'H';	
	
-------------------------------------------------------------------
-- Component Mapping
-------------------------------------------------------------------

	u_System2 : System2
	Port map(
		SysClk			=> SysClk		    ,
		RstB			=> RstB			    ,

		sd_data			=> SdrDq		    ,
		sd_add			=> SdrAddr		    ,
		sd_ras 			=> SdrRas		    ,
		sd_cas 			=> SdrCas		    ,
		sd_we 			=> SdrWe		    ,
		sd_ba			=> SdrBa		    ,
		Clk_Sdram 		=> SdrClk		    ,
		sd_cke 			=> SdrCke		    ,
		sd_cs			=> SdrCs		    ,
		sd_dqm			=> SdrDqm			,
		
		scl				=> scl				,
		sda				=> sda				,

		VGA_Clk			=> VGA_Clk			,
		VGA_nRst		=> VGA_nRst			,
		VGA_PixClk		=> VGA_PixClk		,
		VGA_nPixClk		=> VGA_nPixClk		,
		VGA_Hsync		=> VGA_Hsync		,
		VGA_Hsync_out	=> VGA_Hsync_out	,
		VGA_Vsync		=> VGA_Vsync		,
		VGA_Vsync_out	=> VGA_Vsync_out	,
		VGA_Blank		=> VGA_Blank		,
		VGA_Red			=> VGA_Red			,
		VGA_Green		=> VGA_Green		,
		VGA_Blue 		=> VGA_Blue 	
	);
	
	u_SdRam : mt48lc16m16a2
	Port map(
		Clk	  			=> SdrClk  			,

		Addr  			=> SdrAddr 			,
		Ba	  			=> SdrBa   			,
		Cke	  			=> SdrCke  			,
		Cs_n  			=> SdrCs  			,
		Ras_n 			=> SdrRas 			,
		Cas_n 			=> SdrCas 			,
		We_n  			=> SdrWe  			,
		Dqm	  			=> SdrDqm 			,
		Dq	  			=> SdrDq
	);
	
	u_i2c_slave_model : i2c_slave_model
	Port map(
		scl				=> scl				,
		sda				=> sda
	);
	
-------------------------------------------------------------------
-- Concurrent Signal
-------------------------------------------------------------------

	u_SysClk : Process
	Begin
		SysClk 	<=	'0';
		wait for period/2;
		SysClk		<=	'1';
		wait for period/2;
	End Process u_SysClk;
	
	u_VGAClk : Process
	Begin
		VGA_Clk	<=	'0';
		wait for 2*period;
		VGA_Clk	<=	'1';
		wait for 2*period;			
	End Process u_VGAClk;
	
	u_RstB : Process
	Begin
		RstB	<=	'0';
		wait for 10*period;
		RstB	<=	'1';
		wait;
	End Process u_RstB;
	
-------------------------------------------------------------------
-- Testbench
-------------------------------------------------------------------

	u_Test : Process
	Begin
		
		wait for 10000000*period;
		report("##### End Simulation #####") Severity failure;
	
	End Process u_Test;

End Architecture test;